#!/bin/bash
# search for "main program" to find where this script really starts

unset CDPATH  # CDPATH causes every "cd" command to print the name of the target
	      # directory.

exec 2> /tmp/log
set -x

#######################################################
# 
#   ScriptTableViewer application script
#
#   browse processes
#

##################################################
# set default program state variables
##################################################

STATE=ps    # display the list of processes
	    # Note that state names refer to transition points, not the stuff
	    # currently being displayed.  The 'ps' state is the transition between
	    # nothing being displayed and the directory being displayed.
PID=""
FILENAME=""
DIR=""
TITLE="Process List"

printProgramVariables()
{
   echo "=%==%--STATE=$STATE"
   echo "=%==%--PID=$PID"
   echo "=%==%--DIR=$DIR"
   echo "=%==%--FILENAME=$FILENAME"
   echo "=%==%--TITLE=$TITLE"
}


################################################################
# detect current state from command line.  command line options
# defined like this populate the new state:
#
#  --STATE_VARIABLE_NAME="value"
#
################################################################

while [ $# != 0 ]
do
   # parse options

   case "$1" in
     --*=*)

	  ;;     # process options beginning with --name=
     *)
	  break  # quit when the option does not begin with --
   esac

   name="${1/%=*/}"  # remove =value from a --name=value pair
   value="${1/#*=/}" # remove --name from the --name=value pair

   name=${name:2}    # remove leading --

   eval "$name=\"$value\""

   shift
done

###################################################################################
# library of command handler functions
###################################################################################

handle_ps()
{
    STATE=processes
    printProgramVariables "$@"

    echo "|UID|PID|PPID|C|STIME|TTY|TIME|CMD"
    ps -ef | expand |
    {
       read trash # discard first line
       cat
    } |
    sed -e 's/^ *\([^ ]\+\) *\([^ ]\+\) *\([^ ]\+\) *\([^ ]\+\) *\([^ ]\+\) *\([^ ]\+\) *\([^ ]\+\) *\(.*\)/\1|\2|\3|\4|\5|\6|\7|\8/1'
    
}

handle_processes()
{
    #
    #  print the files in the /proc/$PID
    #
    #  switch to a state wherein members of the list can be selected
    #

    STATE=pid
    PID=$8
    DIR="/proc/$PID"
    TITLE="Process $PID"
    printProgramVariables "$@"

    echo "|Listing process $PID"
    ls -l /proc/$PID
}

handle_pid()
{
    #
    #  given that we are in a state where the user has selected a file in 
    #  /proc/$PID, show that file
    #
    #  switch to a state indicating that we are in the file browser
    #


    FILENAME=`
	       echo $7 |
	       sed -e 's/^ *\([^ ]\+\) *\([^ ]\+\) *\([^ ]\+\) *\([^ ]\+\) *\([^ ]\+\) *\([^ ]\+\) *\([^ ]\+\) *\(.*\)/\8/1' \
		   -e 's/ -> .*//g'
	     `

   TITLE="$DIR/$FILENAME"

   if [ -d "$DIR/$FILENAME" -a -r "$DIR/$FILENAME" ]
   then
      STATE=pid
      DIR="$DIR/$FILENAME"
   else
      STATE=file
   fi

    printProgramVariables "$@"


    if [ "$STATE" = "pid" ]
    then

	# we have a readable directory

        echo "|Listing of /proc/$PID/$FILENAME"
	ls -l "/proc/$PID/$FILENAME/."
    else

	# its not a directory or its not readable

	if [ -r "$DIR/$FILENAME" ]
	then

	    # its readable, so its a file

	    echo "|Listing of $DIR/$FILENAME"
	    cat "$DIR/$FILENAME"
	else
	    echo "|Error Message"
	    echo "Error: /proc/$PID/$FILENAME is not readable" 
	    echo ""
	    echo "   `ls -l /proc/$PID/$FILENAME`"
	fi
    fi

}


##############################################################################
# main program
##############################################################################

case "$STATE" in
    ps)
	handle_ps "$@"
	;;

    processes)
	handle_processes "$@"
	;;

    pid)
	handle_pid "$@"
	;;

    *)
	echo "|Error message"
	echo ""
	echo "     unexpected STATE=$STATE"
	let count=1
	for f in "$@"
	do
	   echo "parm $count: $f"
	   let count=$count+1
	done
	exit 1
esac

exit 0
