#!/bin/bash

unset CDPATH  # CDPATH causes every "cd" command to print the name of the target
	      # directory.

# search for "main program" to find where this script really starts

exec 2> /tmp/log
set -x

#######################################################
# 
#   ScriptTableViewer application script
#
#   It provides a directory listing style interface for
#   a .zip file
#

##################################################
# set default program state variables
##################################################

STATE=dir   # display the zip files in the current directory
	    # Note that state names refer to transition points, not the stuff
	    # currently being displayed.  The 'dir' state is the transition between
	    # nothing being displayed and the directory being displayed.
ARCHIVE=""
TITLE=""
FILENAME=""
SEPCHAR='`'


printProgramVariables()
{
   echo "=%==%--STATE=$STATE"
   echo "=%==%--ARCHIVE=$ARCHIVE"
   echo "=%==%--TITLE=$TITLE"
   echo "=%==%--FILENAME=$FILENAME"
}

if [ "$TITLE" = "" ]
then
    TITLE="Listing Archives in `pwd`"
fi


################################################################
# detect current state from command line.  command line options
# defined like this populate the new state:
#
#  --STATE_VARIABLE_NAME="value"
#
################################################################

while [ $# != 0 ]
do
   # parse options

   case "$1" in
     --*=*)

	  ;;     # process options beginning with --name=
     *)
	  break  # quit when the option does not begin with --
   esac

   name="${1/%=*/}"  # remove =value from a --name=value pair
   value="${1/#*=/}" # remove --name from the --name=value pair

   name=${name:2}    # remove leading --

   eval "$name=\"$value\""

   shift
done

###################################################################################
# library of command handler functions
###################################################################################

handleDirState()
{
    #
    #  You should only get here the first time that the script is invoked
    #  in a given directory (well, actually within a given window in this directory).
    #
    #  We are showing the names of archive files in the current directory in
    #  this script.  If the user clicks on a line, then the archive's index
    #  will be displayed next.  The "next state" after dir is "list".  This 
    #  is the list of members of a given archive.

    if [ $# != 0 ]
    then
	echo "$SEPCHAR""Error Message"
	echo ""
	echo "Error:  unexpected command line arguments in STATE=dir"
	echo ""
	for f in "$@"
	do
	   echo " $f"
	done


    fi

    STATE=list
    printProgramVariables "$@"

    echo "$SEPCHAR""Info""$SEPCHAR""Archive"

    echo -n "$SEPCHAR" | od -x
    
    files=`file * | grep 'Zip archive data' | sed -e 's/^\([^:]\+\):.*/\1/1'`
    
    ls -l1 $files | 
    sed -e 's/^ *\([^ ]\+\) *\([^ ]\+\) *\([^ ]\+\) *\([^ ]\+\) *\([^ ]\+\) *\([^ ]\+\) *\([^ ]\+\) *\(.*\)/\1 \2 \3 \4 \5 \6 \7`\8/1' \
	-e '$s/.*/[!_]&/1'

    

}

handleListState()
{
    # you should only get here after the user has selected on a line in STATE=list
    #
    # here we are showing the names of the members of a specific archive.
    #
    # The next state after this is index (yeah, the name is badly chosen).
    #
 

    if [ $# != 8 ]
    then
       echo "$SEPCHAR""Error Message in STATE=list"
       echo ""
       echo "  Expected 8 command line parameters, found: "
       echo ""
       for f in "$@"
       do
	  echo "  $f"
       done
    else
       STATE=index
       ARCHIVE="$8"
       TITLE="Members of archive $ARCHIVE"
       printProgramVariables "$@"
       echo "$SEPCHAR""Listing"
       echo ""
       unzip -l "$ARCHIVE"
    fi

}

handleIndexState()
{
    # You should only get here after the user has selected on a line in STATE=index
    #
    # In the index state, we are about to show a file selected from the list of
    # members in the archive.  If the member list 

    if [ $# != 7 ]
    then
       echo "$SEPCHAR""Error Message in STATE=index"
       echo ""
       echo "  Expected 7 command line parameters, found: "
       echo ""
       for f in "$@"
       do
	  echo "  $f"
       done
    else
       STATE=file
       FILENAME=`echo "$6" | cut -c29-999`
       printProgramVariables "$@"
       echo "$SEPCHAR""$FILENAME"
       echo ""

       tmpfile=/tmp/zip-browse.$$

       unzip -c "$ARCHIVE" "$FILENAME" |
       {
	  read junk   # remove first to lines of output
	  read junk
	  cat

       }  >$tmpfile

       filetype=` file $tmpfile | tr '[a-z]' '[A-Z]' `

       case  "$filetype" in
	   *TEXT*)
	       cat $tmpfile
	       ;;
	   *)  echo "$filetype"
	       ;;
       esac

       rm $tmpfile
	  
    fi

}


###################################################################################
# main program
###################################################################################

case "$STATE" in

    dir)
	handleDirState "$@"
	;;

    list)
	handleListState "$@"
	;;

    index)
	handleIndexState "$@"
	;;

    file)
	echo "$SEPCHAR""Error message"
	echo ""
	echo "No where to go from a file"
	;;

    *)
	echo "$SEPCHAR""Error message"
	echo "Unknown program state '$STATE'"
	exit 1
	;;
esac
