#include <iostream>           
#include <stdlib.h>

#include <cxxtls/cpptagdb.h>
#include <cxxtls/options.h>
#include <cxxtls/foreach.h>

using namespace std;


#include <stdio.h>

bool somethingDisplayed=false;


int main(int argc, char **argv, char **environ)
{

  cxxtls::ProgramOptions ops("-f,-s,-d,-h,--help,-fm:-cm:-CM:-d,-fr:",
                             argc,
                             argv,
                             environ
                            );


  if(ops.option("-h") || ops.option("--help") || ops.argv.size() < 2)
  {
      cerr <<"\ndumpCppTagDb.exe -- display the cpptagdb.exe generated database." << endl;
      cerr << endl;
      cerr <<"  Usage: " << endl;
      cerr << endl;
      cerr <<"     dumpCppTagDb.exe [options] databaseFile" << endl;
      cerr <<"  Options:" << endl;    
      cerr <<"   -s      show all symbol information" << endl;
      cerr <<"   -f      list all files containing symbols" << endl;
      cerr <<"   -fm n   print files matching name n" << endl; 
      cerr <<"   -fr r1  find file r1 relative to dirs mentioned in symbols" << endl;
      cerr <<"   -d      list all directories definining symbols" << endl;
      cerr <<"   -h      print help" << endl;
      cerr <<"   --help  print help" << endl;
      cerr <<"   -CM m1  print class members matching m1 (case sensitive)" << endl;
      cerr <<"   -cm m2  print class members matching m2 (case insensitive)" << endl;
      cerr << endl;
      exit(0);
  }


    int line;

    cxxtls::CppTagDB::Info info;


    int err = info.populate(ops.argv[1], line);

    if(err)
    {
        cerr << "Error:  " << err << " at line " << line << " in file '" << ops.argv[1] << "'" << endl;
        exit(1);
    }
    else
    if(ops.option("-f"))
    {
        CXXTLS_FOREACH(cxxtls::CppTagDB::Info::FilesMap::value_type const &cur, info.files_)
        {
           cout << cur.second << endl;
        }
    }
    else
    if(ops.option("-fm"))
    {
        string fileNodeName = *ops.option("-fm").begin(); // parm requires a parameter

        vector<string> matches;

        info.findMatchingFiles(fileNodeName, matches);

        cout << "files matching " << fileNodeName << " are:" << endl;

        CXXTLS_FOREACH(string const &cur, matches)  cout << "  " << cur << endl;



    }
    else
    if(ops.option("-CM"))
    {
        string memberName = *ops.option("-CM").begin(); // parm requires a parameter

        vector<cxxtls::CppTagDB::SymbolInfo const *> matches;

        info.findMatchingMembers(memberName, matches);

        cout << "Out of " << info.symbols_.children_.size() <<

        " symbols, the following Symbols match " << memberName << " are:" << endl;

        CXXTLS_FOREACH(cxxtls::CppTagDB::SymbolInfo const *cur, matches)  
            cout << "  " << cur->fullname_ << endl;



    }
    else
    if(ops.option("-cm"))
    {
        string memberName = *ops.option("-cm").begin(); // parm requires a parameter

        vector<cxxtls::CppTagDB::SymbolInfo const *> matches;

        info.findMatchingMembers(memberName, matches, false);

        cout << "Out of " << info.symbols_.children_.size() <<

        " symbols, the following Symbols match " << memberName << " are:" << endl;

        CXXTLS_FOREACH(cxxtls::CppTagDB::SymbolInfo const *cur, matches)  
            cout << "  " << cur->fullname_ << endl;



    }
    else 
    if(ops.option("-d"))
    {

        CXXTLS_FOREACH(string const &cur, info.directories_)  cout << "  " << cur << endl;


    }
    else
    if(ops.option("-s"))
    {

        cout << info << endl;

    }
    else
    if(ops.option("-fr"))
    {
        string relativePath = *ops.option("-fr").begin(); // parm requires a parameter

        vector<string> matches;

        info.findRelativeFiles(relativePath, matches);

        cout << "Matching files are:" << endl;

        CXXTLS_FOREACH(string const &cur, matches)  
            cout << "  " << cur << endl;

    }
    else
    {
       cerr << "Unknown option:  you can't get here from there" << endl; 
    }
}
