//
// Copyright 2002, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.

//@File
//
///  This file defines a thin wrapper around ftp that lets you
///  issue commands from a script.  This wrapper is very insecure because
///  you must supply your password on the command line.


#include <cxxtls/ftp.h>
#include <cxxtls/options.h>
#include <stdlib.h>

using namespace std;
using namespace cxxtls;

int main(int argc, char **argv, char **environ)
{
  ProgramOptions op("-stat,-get,-put,-del", argc, argv, environ);

  int status = 0;
  int get    = 0;
  int put    = 0;
  int del    = 0;

  if(op.option("-stat"))
    status = 1;

  if(op.option("-get"))
    get = 1;

  if(op.option("-put"))
    put = 1;

  if(op.option("-del"))
    del = 1;

  if( (get + del + put + status) > 1 )
  {
    cerr << "only one of -get, -put, -del, or -stat can be issued at a time" << endl;
    exit(1);
  }


  if(   (    (!get && !put) 
          && (op.argv.size() != 5) 
        ) 
     || (   ( (get  ||  put) 
              && (op.argv.size() < 5)
            )
         || (op.argv.size() > 6) 
        )
    )
  {
    cerr << "Error:  expected exactly 4 non-option parameters" << endl
	 << "        hostname" << endl
	 << "        username" << endl
	 << "        password" << endl
	 << "        file search pattern" << endl
	 << ((get || put ) ? "        [localfile]" : "" ) << endl
	 ;
	
    cerr << "        Options are " << endl
	 << "	       -stat  -- display file status as	well as	the names " << endl
	 << "	       -get   -- fetch a remote	file " << endl
	 << "	       -put   -- copy a	local file to the remote system	" << endl
	 << "	       -del   -- delete	a file on the remote system " << endl;
	
    exit(1);

  }

  if(status)
  {
    list<FTP::FileInfo> lf;

    int count = FTP::stat_matching(op.argv[1],
				   op.argv[2],
				   op.argv[3],
				   op.argv[4],
				   &lf
				  );
				
   if(count >= 0)
   {
     list<FTP::FileInfo>::const_iterator i = lf.begin();

     while( i != lf.end() )
     {
       cout << i->status_ << " " << i->name_ << endl;
       ++i;
     }

     cout << count << " matches found" << endl;
   }
   else
   {
     cout << FTP::error() << endl;
   }
  }
  else
  if(get)
  {
    string tmp;

    if(op.argv.size() == 6)
      tmp = op.argv[5];

    if(FTP::get(op.argv[1],op.argv[2],op.argv[3],op.argv[4], tmp))
    {
      cerr << FTP::error() << endl;
    }
  }
  else
  if(del)
  {
    if(FTP::del(op.argv[1],op.argv[2],op.argv[3],op.argv[4]))
    {
      cerr << FTP::error() << endl;
    }
  }
  else
  if(put)
  {
    string tmp;

    if(op.argv.size() == 6)
      tmp = op.argv[5];

    if(FTP::put(op.argv[1],op.argv[2],op.argv[3],op.argv[4], tmp))
    {
      cerr << FTP::error() << endl;
    }
  }
  else
  {
    list<FTP::FileInfo> lf;

    int count = FTP::stat_matching(op.argv[1],
				   op.argv[2],
				   op.argv[3],
				   op.argv[4],
				   &lf
				  );
				
   if(count >= 0)
   {
     list<FTP::FileInfo>::const_iterator i = lf.begin();

     while( i != lf.end() )
     {
       cout << i->name_ << endl;
       ++i;
     }

     cout << count << " matches found" << endl;

   }
   else
   {
     cout << FTP::error() << endl;
   }
  }

  exit(0);

}
