//
// Copyright 2013, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.


//@File
// This file is a temporary development process helper.  It lets me run the
// GdbController class on a real program being debugged


#ifdef __linux__
#include <cxxtls/gdbcontroller.h>
using namespace cxxtls;
#endif


#include <iostream>

using namespace std;



int main(int argc, char **argv)
{
#ifndef __linux__
  cout << "OOh!  No no no no no!  No can do big guy!  No way, Jose Cuervo" << endl;
#else

  CXXTLS_SET_PROGRAM_TRACE_LEVEL(cxxtls::TT_DEBUG);


  string mydir = argv[0];

  mydir = mydir.substr(0,  mydir.find_last_of('/') );

  if(mydir == argv[0])
     mydir = ".";

  string debuggee = mydir + "/cpptagdb.exe";

  string parms    = mydir + "/*.cxx";

  GdbController  gdb(true, "gdblog.log");

  gdb.setExecutable(debuggee);
  gdb.addBreakpoint("main");
  gdb.setArgs(parms);

  CXXTLS_LOG("ABOUT TO RUN TO MAIN");

  gdb.run();

  sleep(1);

  // we should be sitting at main now

  if(int(gdb.state()) != int(GdbController::stopped))
  {
     CXXTLS_DEBUG("Error:  in state " << gdb.stateName() << " but expected to be in " 
                  << "GdbController::stopped"
                 );
          
  }

  CXXTLS_LOG("Current Gdb state = " << gdb.stateName());


  CXXTLS_LOG(">>>>>>>>>>>>>>>>>>>>>>>\n"
             << "Program Output:");
  gdb.printAndDeleteTargetOutput();
  CXXTLS_LOG(">>>>>>>>>>>>>>>>>>>>>>>");

  CXXTLS_LOG("ABOUT TO PRINT THREADS");

  GdbController::IdList threadNames;

  gdb.threads(threadNames);

  CXXTLS_FOREACH(std::string const &cur, threadNames)
  {
     std::string threadInfo = gdb.thread(cur);

     CXXTLS_LOG("   thread(" << cur << "):  " << threadInfo);


  }

  // here we should be stopped at main().  Add some display variables
  CXXTLS_LOG("ABOUT TO ADD DISPLAY EXPRESSIONS");

  string argcVariableName        = gdb.addMonitoredVariable("argc");
  string argvVariableName        = gdb.addMonitoredVariable("argv");
  string testvarVariableName     = gdb.addMonitoredVariable("testvar");
  string testCPVariableName      = gdb.addMonitoredVariable("testCP");
  string testMCPVariableName     = gdb.addMonitoredVariable("testMCP");
  string testMArrayVariableName  = gdb.addMonitoredVariable("testMArray");
  string testArrayariableName    = gdb.addMonitoredVariable("testArray");


  sleep(1);
  CXXTLS_LOG("ABOUT TRIGGER A LISTING OF THE THREADS");

  gdb.sendCommand("-thread-info");

  sleep(1);
  CXXTLS_LOG("ABOUT TO STEP OVER THE FIRST LINE IN MAIN");


  gdb.stepOver();

  sleep(1);

  CXXTLS_LOG("ABOUT TO print testvar._M_p");

  gdb.triggerMonitoredVariableEvaluation("D12.private._M_dataplus.public._M_p");

  sleep(1);

  CXXTLS_LOG("\nAfter Step-over of one line, we see:  Gdb state = " << gdb.stateName());

  CXXTLS_LOG( ">>>>>>>>>>>>>>>>>>>>>>>"  );
  CXXTLS_LOG( "Program Output:" );
  gdb.printAndDeleteTargetOutput();
  CXXTLS_LOG( ">>>>>>>>>>>>>>>>>>>>>>>"  );

  CXXTLS_LOG("ABOUT TO CHANGE ARGC");

  gdb.changeExpression(argcVariableName, "10");

  CXXTLS_LOG("ABOUT TO STEP INTO THE SECOND LINE IN MAIN");

  gdb.stepInto();

  sleep(1);

  CXXTLS_LOG( "\nAfter Step-Into of one line, we see:  Gdb state = " << gdb.stateName() );

  CXXTLS_LOG( ">>>>>>>>>>>>>>>>>>>>>>>"  );
  CXXTLS_LOG( "Program Output:" );
  gdb.printAndDeleteTargetOutput();
  CXXTLS_LOG( ">>>>>>>>>>>>>>>>>>>>>>>"  );



  CXXTLS_LOG("ABOUT TO ADD A BREAKPOINT:  parse_file()");

  gdb.addBreakpoint("parse_file");
  sleep(1);

  CXXTLS_LOG("ABOUT TO TOGGLE BREAKPOINT ON/OFF parse_file");

  std::list<std::string> bpnames;

  gdb.breakpoints(bpnames);

  CXXTLS_FOREACH(std::string const &cur, bpnames)
  {
      std::string info = gdb.breakpoint(cur);

      CXXTLS_LOG("    bp:  " << info);
  }

  CXXTLS_LOG("ABOUT TO CLEAR ALL BREAKPOINTS");

  CXXTLS_FOREACH(std::string const &cur, bpnames)
  {
      gdb.removeBreakpoint(cur);
  }
  sleep(1);

  gdb.breakpoints(bpnames);

  if(!bpnames.empty())
  {
     CXXTLS_LOG("Error:  could not remote these breakpoints: ");

     CXXTLS_FOREACH(std::string const &cur, bpnames)
     {
         std::string info = gdb.breakpoint(cur);
         CXXTLS_LOG("    bp:  " << info);
         
     }

  }

  CXXTLS_LOG("ABOUT TO PRINT ALL DISPLAY EXPRESSIONS");

  gdb.stepOut();

  sleep(1);

  GdbController::IdList dispVars;

  gdb.variables(dispVars);

  gdb.logMonitoredVariables();

  CXXTLS_LOG("ABOUT TO REMOVE A DISPLAY EXPRESSION (argv)");

  gdb.removeMonitoredVariable(argvVariableName);

  gdb.variables(dispVars);

  CXXTLS_LOG("ABOUT TO RE-PRINT ALL DISPLAY EXPRESSIONS");

  gdb.logMonitoredVariables();

  CXXTLS_LOG("ABOUT TO CONTINUE TO END OF PROGRAM");

  gdb.resume();

  sleep(2);

  CXXTLS_LOG( "\n\nDone!  Exiting.  Gdb state = " << gdb.stateName() );

  CXXTLS_LOG( ">>>>>>>>>>>>>>>>>>>>>>>"  );
  CXXTLS_LOG( "Program Output:" );
  gdb.printAndDeleteTargetOutput();
  CXXTLS_LOG( ">>>>>>>>>>>>>>>>>>>>>>>"  );


#endif


}

