#include <cxxtls/foreach.h>
#include <stdio.h>
#include <vector>
#include <iostream>
#include <list>
#include <map>
#include <set>
#include <deque>
#include <string>


void myprint(char const *msg)
{
   printf("%s\n", msg);
}

int main()
{
    using namespace std;

    //
    //  Test all the cases that are SUPPOSED to compile.
    //

    //==============================================================
    //
    // 1.  Simple non-const std::vector
    //
    //==============================================================

    cxxtls::foreach_helpers::AnyType<int> anInt(4);

    vector<int> vi;

    vi.push_back(1);
    vi.push_back(2);
    vi.push_back(3);

    cout << "about to start\n" << endl;

    int value=0;
    myprint("first begins");
    CXXTLS_FOREACH(int curItem, vi)
    {
       cout << "curItem is " << curItem << endl;
       value += curItem;
    }
    myprint("first ends");

    cout << "sum = " << value << endl;


    //==============================================================
    //
    // 2.  Simple const std::vector
    //
    //==============================================================

    vector<int> const & cvi = vi;

    CXXTLS_FOREACH(int curItem, cvi)  cout << "const curItem is " << curItem << endl;


    //==============================================================
    //
    // 3.  const array of const character strings
    //
    //==============================================================

    static char const * const array[] =
    {
       "word1",
       "word2",
       "word3"
    };

    cxxtls::foreach_helpers::AnyTypeBase const &container = 
                                       cxxtls::foreach_helpers::getContainer(array);

    cxxtls::foreach_helpers::AnyType< cxxtls::foreach_helpers::IteratorWrapper<char const * *> > array_begin = 
                                  cxxtls::foreach_helpers::getBegin(container, CXXTLS_NULL_TYPE_PTR(array) );

    cout << "array begin = " << *array_begin.value_.iterator_ << endl;


    cxxtls::foreach_helpers::AnyType< cxxtls::foreach_helpers::IteratorWrapper<char const * *> > array_end = 
                                   cxxtls::foreach_helpers::getEnd(container, CXXTLS_NULL_TYPE_PTR(array) );

    --array_end.value_.iterator_;  // point to last valid item

    cout << "array end = " << *array_end.value_.iterator_ << endl;

    cout << "deref(current) = " << cxxtls::foreach_helpers::deref(array_begin, CXXTLS_NULL_TYPE_PTR(array) ) << endl;

    cout << "continuing(beg,end) = " 
         <<  cxxtls::foreach_helpers::continuing(array_begin, array_end, CXXTLS_NULL_TYPE_PTR(array)) << endl;

    cout << "continuing(end,end) = " 
         <<  cxxtls::foreach_helpers::continuing(array_end, array_end, CXXTLS_NULL_TYPE_PTR(array)) << endl;

    char const *second = "glarf";
    
    cxxtls::foreach_helpers::next(array_begin, array_end, CXXTLS_NULL_TYPE_PTR(array));

    second=cxxtls::foreach_helpers::deref(array_begin, CXXTLS_NULL_TYPE_PTR(array));

    cout << "after begin is " << second << endl;

    CXXTLS_FOREACH(char const *garfItem, array)  cout << "const curItem is " << garfItem << endl;

    //==============================================================
    //
    // 4.  array of const int
    //
    //==============================================================

    static const int int_array[]=
    {
      100,
      200,
      300
    };

    int int_array_sum=0;

    CXXTLS_FOREACH(int const &cur, int_array)  int_array_sum += cur;

    cout << "int array sum is " << int_array_sum << endl;


    //==============================================================
    //
    // 5.  map<int,int>
    //
    //==============================================================

    typedef map<int,int> MapType;

    MapType mi;

    mi[0]=0;
    mi[1]=1;
    mi[2]=2;
    mi[3]=3;

    int_array_sum = 0;
    int loop_count=0;

    CXXTLS_FOREACH(MapType::value_type const &cur, mi)  
    {
        int_array_sum += cur.first;
        ++loop_count;
    }
    
    cout << "sum of map keys is " << int_array_sum << endl;
    cout << "count map loop iterations  is " << loop_count << endl;


    //==============================================================
    //
    // 6.  Validate simple std::set<> operations
    //
    //==============================================================


    typedef set<int> SetType;

    SetType si;  
    
    si.insert(200); 
    si.insert(300); 
    si.insert(400);

    int_array_sum = 0;
    loop_count=0;

    CXXTLS_FOREACH(SetType::value_type const & cur, si)  
    {
        int_array_sum += cur;
        ++loop_count;
    }
    
    cout << "sum of set keys is " << int_array_sum << endl;
    cout << "count set loop iterations  is " << loop_count << endl;


    //==============================================================
    //
    // 7.  Validate simple std::deq<> operations
    //
    //==============================================================


    typedef deque<int> DequeType;

    DequeType di;  
    
    di.push_back(200); 
    di.push_back(300); 
    di.push_back(400);

    int_array_sum = 0;
    loop_count=0;

    CXXTLS_FOREACH(DequeType::value_type const & cur, di)  
    {
        int_array_sum += cur;
        ++loop_count;
    }
    
    cout << "sum of deque values " << int_array_sum << endl;
    cout << "count deque loop iterations  is " << loop_count << endl;


    int_array_sum = 0;
    loop_count=0;

    CXXTLS_FOREACH(DequeType::value_type & cur, di)  
    {
        int_array_sum += cur;
        ++loop_count;
    }
    
    cout << "sum of deque values " << int_array_sum << endl;
    cout << "count deque loop iterations  is " << loop_count << endl;


    //==============================================================
    //
    // 8.  Validate simple std::string<> operations
    //
    //==============================================================

    string stri;  
    
    stri.push_back(char(1)); 
    stri.push_back(char(2)); 
    stri.push_back(char(3));

    int_array_sum = 0;
    loop_count=0;

    CXXTLS_FOREACH(string::value_type const & cur, stri)  
    {
        int_array_sum += cur;
        ++loop_count;
    }
    
    cout << "sum of string values " << int_array_sum << endl;
    cout << "count string loop iterations  is " << loop_count << endl;

    {
       cout << "Testing break and continue" << endl;


       int array[3];

       array[0] = 1;
       array[1] = 2;
       array[2] = 3;

       loop_count = 0;

       CXXTLS_FOREACH(int element, array)
       {
         element= element;  // stop warning
         loop_count++;
       }

       cout << "Iterating over an array of 3 gave a loop count of " << loop_count << endl;

       loop_count = 0;

       CXXTLS_FOREACH(int element, array)
       {
         if(element==2)
            continue;

         loop_count++;
       }

       cout << "Iterating over an array of 3 but skipping one gave a loop count of " << loop_count << endl;

       loop_count = 0;

       CXXTLS_FOREACH(int element, array)
       {
         if(element==2)
            break;

         loop_count++;
       }

       cout << "Iterating over an array of 3 but stopping after one gave a loop count of " << loop_count << endl;



    }

   cout << "done\n" << endl;

}





