
//
// Copyright 2010, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.
//

#include <BIscan/BIscan.h>
#include <iostream>
#include <stdlib.h>

#define BOMB(msg)  { cout << __FILE__ << ":" << __LINE__ << ":  " << msg << endl; exit(1); }


using namespace std;

using namespace BIscan;

int main()
{
   int value, value2;

   if( convertInteger("", value) )
   {
      BOMB("Error:  expected empty string to fail to convert to integer");
   }

   if(!convertInteger("0", value) || value != 0)
   {
      BOMB("Error:  expected '0' to convert to 0");
   }

   if(!convertInteger("1", value) || value != 1)
   {
      BOMB("Error:  expected '1' to convert to 1");
   }

   if(!convertInteger("-1", value) || value != -1)
   {
      BOMB("Error:  expected '-1' to convert to -1");
   }

   if(!convertInteger("    -1", value) || value != -1)
   {
      BOMB("Error:  expected '    -1' to convert to -1");
   }

   if(!convertInteger("10", value) || value != 10)
   {
      BOMB("Error:  expected '10' to convert to 10");
   }

   if(!convertInteger("-10", value) || value != -10)
   {
      BOMB("Error:  expected '-10' to convert to -10");
   }

   char const *badNumber = "    -crap";
   char const *goodNumber= "    -19";

   char const *badEnd = badNumber + strlen(badNumber);
   char const *goodEnd= goodNumber + strlen(goodNumber);

   char const *goodPointer = goodNumber;
   char const *badPointer  = badNumber;

   value=-1;

   if(parseInteger(badPointer, badEnd, value))
   {
      BOMB("convertInteger('  -crap') should not have converted");
   }
   else
   {
     if(badPointer != badNumber)
     {
         BOMB("convertInteger('  -crap') should not have moved the start pointer");
     }

     if(value != -1)
     {
         BOMB("convertInteger('  -crap') should not have set a return value");
     }

   }

   if(!parseInteger(goodPointer, goodEnd, value))
   {
      BOMB("convertInteger('  -19') should not have converted");
   }
   else
   {
     if(goodPointer != goodEnd)
     {
         BOMB("convertInteger('  -19') should have moved the start pointer to the end");
     }

     if(value != -19)
     {
         BOMB("convertInteger('  -19') should have detected -19");
     }

   }

   if( !convertInteger("1001", value, 2, false) || value != 9)
   {
         BOMB("convertInteger('1001',2,false) should have detected 9");
   }

   if( !convertInteger("-11", value, 2, true) || value != -3)
   {
         BOMB("convertInteger('-11',2,true) should have detected -3");
   }

   if( !convertInteger("20", value, 16, false) || value != 32)
   {
         BOMB("convertInteger('20',16,false) should have detected 32");
   }

   string intValueText("    -37");

   if(!convertInteger(intValueText, value) || value != -37)
   {
         BOMB("convertInteger(std::string) should have detected -37");
   }

   {
      double d;
      float  f;

      if(!convertDouble("  13", d) || d != 13)
      {
        BOMB("cannot convert ' 13' to double");
      }

      if(!convertFloat("  13", f) || f != 13)
      {
        cout << "f is " << f << endl;
        BOMB("cannot convert ' 13' to float");
      }

      std::string twopointtwo("  22");

      if(!convertDouble(twopointtwo, d) || d != 22)
      {
        BOMB("cannot convert ' 22' to double");
      }

      if(!convertFloat(twopointtwo, f) || f != 22)
      {
        BOMB("cannot convert ' 22' to float");
      }



   }


   
   {
   
       int variable, a, b, c, h, o, bin;

       double d1=-1;
       float  f1=-2;
    
       std::string crap;
    
       std::string other;
    
       if( BIscan::stream("1 2 3 4  crap other 10 10 10 1e5 -9.2e7") >> variable 
                                                     >> a 
                                                     >> b 
                                                     >> c 
                                                     >> crap 
                                                     >> other 
                                                     >> Hex(h)  
                                                     >> Octal(o)  
                                                     >> Binary(bin)
                                                     >> d1
                                                     >> f1
         )
       {
          cout << "BIscan::stream parsing returned true" << endl;

          if(   variable != 1 
             || a != 2 
             || b != 3 
             || c != 4 
             || h != 16 
             || o != 8 
             || bin != 2
            )
          {
             BOMB("operator >> (BIscan::stream,int) failed");
          }

          if(crap != string("crap") )
          {
             BOMB("operator >> (BIscan::stream,char array[]) failed");
          }

          if(other != "other" )
          {
             BOMB("operator >> (BIscan::stream,std::string) failed");
          }

       }
       else
       {
          BOMB("Could not parse '1 2 3  4 crap other 10 10 10 1e5 -9.2e7'" );
       }

       if( d1 != 1e5 || f1 != -9.2e7 )
       {
          BOMB("floating point conversions failed");
       }
    
       cout << "variable = " << variable << endl;
       cout << "       a = " << a        << endl;
       cout << "       b = " << b        << endl;
       cout << "       c = " << c        << endl;
       cout << "    crap = " << crap     << endl;
       cout << "    other= " << other    << endl;
       cout << "     hex = " << h        << endl;
       cout << "   octal = " << o        << endl;
       cout << "  binary = " << bin      << endl;
       cout << "  double = " << d1       << endl;
       cout << "   float = " << f1       << endl;
    
   }

   std::string buffer;

   if( !(stream( string("stuff 99") ) >> buffer >> value)

      || string("stuff") != buffer
      || value != 99

     )
   {
       BOMB("bad parse in: stream( string('stuff 99') >> buffer >> value");
   }


   if( !stream("glarf") >> value )
   {
      // cout << "Yay! -- stream should have returned failure on this conversion" << endl;
   }
   else
   {
      BOMB("Doh, it did not return false like it should have");
   }

   if( !!stream("14") >> value )
   {
      // cout << "Yay! -- stream should have returned failure on this conversion" << endl;
   }
   else
   {
      BOMB("Doh!  double operator ! did not work!");
   }


   if( !stream("14 , 52") >> value >> SKIPSET(" ,") >> value2)
   {
      BOMB("We should have parsed '14 , 52' correctly");
   }

   if(value != 14 || value2 != 52)
   {
      BOMB("'14 , 52' did not parse correctly");
   }

   if( stream("14x52") >> value >> SKIPSET("X") >> value2)
   {
      BOMB("We should NOT have parsed '14x52' -- we expected X not x");
   }


   if( !stream("99 glarf 114") >> value >> "glarf" >> value2 )
   {
      BOMB("We should have parsed '99 glarf 114'!");
   }

   if( stream("99 glarf 114") >> value >> "flarg" >> value2 )
   {
      BOMB("We should have NOT parsed '99 glarf 114'!  Expected 99 flarg 114");
   }

   {  // some numeric range situations

      unsigned char c=0;

      // you can't use stream to convert a string to a character which treated as an integer.
      // the implementation assumes that if you ask for a character conversion, you want a character
      // conversion.  

      if(!stream("255") >> c || c != (unsigned)(255) )
      {
         BOMB("couldn't convert 255 to an unsigned character, c is " << c );
      }


      // largest (unsigned) -1:  18446744073709551615
      // largest non-negative :  9223372036854775807

                long long neg1_signed;
      unsigned  long long neg1_unsigned;

      if(stream("18446744073709551615") >> neg1_signed)
      {
         if(neg1_signed != -1)
         {
            BOMB("bad conversion of largest unsigned long long into -1");
         }
      }
      else
         BOMB("failed to convert largest unsigned long long into -1");

      if(stream("18446744073709551615") >> neg1_unsigned)
      {
         unsigned long long testValue = -1;

         if(neg1_unsigned != testValue)
         {
            BOMB("bad conversion of largest unsigned long long into -1: "  << neg1_unsigned);
         }
      }
      else
         BOMB("failed to convert largest unsigned long long into -1");


      if(stream("18446744073709551616") >> neg1_unsigned)
      {
         unsigned long long testValue = 0;

         if(neg1_unsigned != testValue)
         {
            BOMB("bad conversion of 1 + largest unsigned long long into 0: "  << neg1_unsigned);
         }
      }
      else
         BOMB("failed to convert 1 + largest unsigned long long into 0");


      if(stream("9223372036854775807") >> neg1_signed)
      {
         if(neg1_signed != 9223372036854775807LL)
         {
            BOMB("bad conversion of largest signed long long into 9223372036854775807");
         }
      }
      else
         BOMB("failed to convert largest signed long long into 9223372036854775807");

      if(stream("9223372036854775807") >> neg1_unsigned)
      {
         unsigned long long testValue = 9223372036854775807LL;

         if(neg1_unsigned != testValue)
         {
            BOMB("bad conversion of largest unsigned long long into 9223372036854775807: "  << neg1_unsigned);
         }
      }
      else
         BOMB("failed to convert largest signed long into unsigned long long into 9223372036854775807");


      short short1_signed, short1_unsigned;

      if(stream("65535") >> short1_signed)
      {
         if( short1_signed != short(-1))
         {
            BOMB("failed to convert 65535 into short(-1):  values are " << short1_signed << " and expected " << short(-1) );
         }
      }
      else
         BOMB("failed to parser 65535 into short");

      if(stream("65535") >> short1_unsigned)
      {
         if( short1_unsigned != short(-1))
         {
            BOMB("failed to convert 65535 into short(-1):  values are " << short1_unsigned << " and expected " << (unsigned short)(-1) );
         }
      }
      else
         BOMB("failed to parser 65535 into unsigned short");

      if(stream("32767") >> short1_signed)
      {
         if( short1_signed != short(32767))
         {
            BOMB("failed to convert 32767 into short(32767):  values are " << short1_signed << " and expected " << short(32767) );
         }
      }
      else
         BOMB("failed to parser 32767 into short");

      if(stream("32767") >> short1_unsigned)
      {
         if( short1_unsigned != short(32767))
         {
            BOMB("failed to convert 32767 into short(32767):  values are " << short1_unsigned << " and expected " << (unsigned short)(32767) );
         }
      }
      else
         BOMB("failed to parser 32767 into unsigned short");


      bool b = false || stream("1") >> value; // just to make sure it compiles

      if(!b)
      {
        BOMB("failed to evaluate operator|| correctly");
      }

   }


   cout << "Yeah, the test passed!" << endl;

   exit(0);

}
