#include <cxxtls/charSetTranslator.h>
#include <iostream>
#include <stdlib.h>

using namespace std;
using namespace cxxtls;


int main()
{
    cout << "charSetTranslator test program" << endl;

    struct TestData
    {
       char const *input;
       char const *lhs;
       char const *rhs;
       char const *expected;
    };

    static TestData data[]=
    {
        { "this stuff"    , "a"      , "a"        , "this stuff" }, // do nothing translation
        { "this stuff"    , "s"      , "S"        , "thiS Stuff" }, // single character translation, no ranges
        { "this stuff"    , "fs"     , "FS"       , "thiS StuFF" }, // 2-single character translations, no ranges
        { "this stuff"    , "sf"     , "SF"       , "thiS StuFF" }, // 2-single character translations, no ranges
        { "abcdefghij"    , "a-c"    , "ABC"      , "ABCdefghij" }, // range on the lhs
        { "abcdefghij"    , "f-jde"  , "F-JDE"    , "abcDEFGHIJ" }, // range on the both
        { "abcdefghij"    , "def-j"  , "DEF-J"    , "abcDEFGHIJ" }, // range on the both
        { "abcdefghij"    , "af-jb"  , "aF-Jb"    , "abcdeFGHIJ" }, // range on the both
        { "abcdefghi-"    , "-f-jb"  , "+F-Jb"    , "abcdeFGHI+" }, // range on the both
        { "a\tb"          , "\t"     , " "        , "a b"        }, // lhs escape sequence testing
        { "abc"           , "b"      , "\t"       , "a\tc"       }, // rhs escape sequence testing
        { "abc"           , "b"      , "\\o011"   , "a\tc"       }, // rhs escape sequence testing
        { "abcABC"        , "a-eA-E" , "1-56789A" , "123678"     }, // range mapping
        { "abcABC"        , "a-fA-F" , "1-61-6"   , "123123"     }, // range mapping
    };

    for(size_t i = 0; i < sizeof(data) / sizeof(data[0]); ++i)
    {
        char const *input    = data[i].input;
        char const *expected = data[i].expected;

        CharSetTranslator  xlate(data[i].lhs, data[i].rhs);

        if(!xlate.ok())
        {
           cerr << "Error:  CharSetTranslator failed due to bad translation criteria:" << endl;
           cerr << "        lhs(" << data[i].lhs << ")" << endl;
           cerr << "        rhs(" << data[i].rhs << ")" << endl;
           cerr << endl;
           cerr << "        reason(" << xlate.error() << ")" << endl;
           exit(1);
        }

        string tmp = xlate.translate(input);

        if(tmp != expected)
        {
           cerr << "Error:  Char set translation failed " << endl;
           cerr << "        lhs(" << data[i].lhs << ")" << endl;
           cerr << "        rhs(" << data[i].rhs << ")" << endl;
           cerr << "        inp(" << input       << ")" << endl;
           cerr << "        out(" << tmp         << ")" << endl;
           cerr << "        EXP(" << expected    << ")" << endl; 
           exit(1);
        }

        cout << "   translate(" << data[i].lhs << "," << data[i].rhs << ") on " << input << " produces " << tmp << endl;

    }

    cout << "    PASSED!" << endl;
}
