
//
// Copyright 2002, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.



#include <algorithm>
#include <cxxtls/cpp_token_stream.h>
#include <string.h>
#include <iostream>
#include <fstream>

using namespace std;
using namespace cxxtls;

class PrepHandler
: public CPP_Token_Stream_Prep
{
public:
  void operator() (std::string const &define,
		   std::string const &file,
		   int                line) const
  {
    cout << "\"" << file << "\", line " << line <<  " " << define << endl;
  }
};


int main()
{
   CPP_Token rv;

   // validate parsing of files using streambuf iterators

   cout << "Parsing the file" << endl;

   istreambuf_iterator<char> cur(cin);
   istreambuf_iterator<char> end;

   PrepHandler ph;

   CPP_Token_Stream< istreambuf_iterator<char>  > cpp_token(cur,end,"stdin", &ph);

   do
   {
     cpp_token(rv);

     cout << rv << endl;

   }
   while(rv.type_ != CPP_Token::eof);

   // validate parsing of character arrays

   static char const* source="one, two+3/4.9";

   cout << "Parsing the character buffer" << endl;

   CPP_Token_Stream< char const* > char_cpp_token(source, source + strlen(source));

   do
   {
     char_cpp_token(rv);

     cout << rv << endl;

   }
   while(rv.type_ != CPP_Token::eof);

   // validate parsing of std::string

   cout << "Parsing the string" << endl;


   std::string s("b+mother::frank_*(c/d)");

   CPP_Token_Stream< std::string::const_iterator > str_cpp_token(s.begin(), s.end());

   do
   {
     str_cpp_token(rv);

     cout << rv << endl;

   }
   while(rv.type_ != 0);


   {
      cout << "Test manually opened stream" << endl;
	
	
      std::ifstream exec_source("cpp_token_test.dat");

      if(!exec_source)
      {
	cerr << "Error, can't open 'cpp_token_test.dat'" << endl;
	exit(1);
	return 0;
      }

      istreambuf_iterator<char> in(exec_source);
      istreambuf_iterator<char> end;


      CPP_Token_Stream< istreambuf_iterator<char> > source(in, end,"cpp_token_test.dat");

      for(;;)
      {
	 CPP_Token t;

	 source(t);

	 cout << t << endl;

	 if(t.type_ == CPP_Token::eof)
	   break;
      }
	
   }

   {
      cout << "Test pre-processor #line intepretation" << endl;
	
	
      std::ifstream exec_source("cpp_token_test.1.dat");

      if(!exec_source)
      {
	cerr << "Error, can't open 'cpp_token_test.1.dat'" << endl;
	exit(1);
	return 0;
      }

      istreambuf_iterator<char> in(exec_source);
      istreambuf_iterator<char> end;


      CPP_Token_Stream< istreambuf_iterator<char> > source(in, end,"cpp_token_test.1.dat");

      for(;;)
      {
	 CPP_Token t;

	 source(t);

	 cout << t << endl;

	 if(t.type_ == CPP_Token::eof)
	   break;
      }
	
   }


   exit(0);

  return 0;
}
