//
// Copyright 2013, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.


//@File
///   This file defines a test for the GdbController class.
///
///   The program defined here, expects the existing of a sister file, gdbTarget3.exe.
///   This file is a program that will debugged using GdbController and
///   the outputs produced from this program will be compared against
///   gdbcontroller_test3.expect
///


#ifndef _MSC_VER
#include <cxxtls/gdbcontroller.h>
using namespace cxxtls;
#endif


#include <iostream>
#include <stdlib.h>

using namespace std;
using namespace cxxtls;


int main(int argc, char **argv)
{
#ifdef _MSC_VER
  cout << "OOh!  No no no no no!  No can do big guy!  No way, Jose Cuervo" << endl;
#else

  CXXTLS_SET_PROGRAM_TRACE_LEVEL(cxxtls::TT_DEBUG);


  string mydir = argv[0];

  mydir = mydir.substr(0,  mydir.find_last_of('/') );

  if(mydir == argv[0])
  {
     mydir=".";  // in this case, the pull pathname was not passed to the executable.
  }

  string debuggee = mydir + "/gdbTarget3.exe";

  GdbController  gdb(true, debuggee + ".gdblog.log");

  gdb.setExecutable(debuggee);
  gdb.addBreakpoint("function");

  gdb.run();

  sleep(1);

  if(gdb.state() != GdbController::stopped)
  {
     CXXTLS_LOG("Error:  expected to have stopped by this time.  Exiting");
     exit(1);
  }

  std::string stopInfoText = gdb.stopInfo();

  std::auto_ptr<VarObj>  stopInfoDeleter;

  VarObj::ParseResult stopRes = VarObj::parse(stopInfoText);

  VarObj *stopInfo = 0;;

  if(stopRes.rv_)
  {
      stopInfoDeleter.reset(stopRes.rv_);
      stopInfo = stopRes.rv_;
  }




  if(!stopInfo)
  {
     CXXTLS_LOG("Error:  expected to have stopped by this time.  Exiting");
     exit(1);
     
  }
  CXXTLS_LOG("Stop Point:  " << stopInfo->value());

  VarObj const *func = stopInfo->findNestedChild("frame/func");

  if(!func)
  {
     CXXTLS_LOG("Error:  expected to have stopped by this time.  Exiting");
     exit(1);
     
  }

  CXXTLS_LOG("Current Stop point:  " << func->value());

  if("function" != func->value())
  {
     CXXTLS_LOG("Error:  did  not stop in the function named 'function'");
     exit(1);
  }

  CXXTLS_LOG("ABOUT TO ADD DISPLAY EXPRESSIONS");

  gdb.stepOver(); // execute the first statement in function "function" in gdbTarget3.cxx.
  sleep(1);       // wait for the running process to stop

  string argcVariableName        = gdb.addMonitoredVariable("x");

  GdbController::IdList names;

  gdb.variables(names);

  CXXTLS_FOREACH(string const &cur, names)
  {
     CXXTLS_LOG("   Declared variable " << cur);
  }

  CXXTLS_LOG("STEPPING OUT TO main()");

  gdb.stepOut();
  sleep(1);

  gdb.variables(names);

  CXXTLS_LOG("SHOULD NOT HAVE VARIABLES IN THE LIST BELOW");


  CXXTLS_FOREACH(string const &cur, names)
  {
     CXXTLS_LOG("   Found variable active in the main() scope:  " << cur);
  }
  CXXTLS_LOG("END OF LIST -- WERE THERE ANY VARIABLES ABOVE?");

  CXXTLS_LOG("RUNNING AGAIN TO function()");

  gdb.resume();  

  sleep(1);

  CXXTLS_LOG("SHOULD FIND THE SAME VARIABLE ACTIVE AGAIN");

  gdb.variables(names);

  CXXTLS_FOREACH(string const &cur, names)
  {
     CXXTLS_LOG("   Found variable active in the main() scope:  " << cur);
  }

  CXXTLS_LOG("END OF LIST -- WHERE THERE ANY VARIABLES ABOVE?");

  gdb.resume();

  CXXTLS_LOG( "\n\nDone!  Exiting.  Gdb state = " << gdb.stateName() );

  CXXTLS_LOG( ">>>>>>>>>>>>>>>>>>>>>>>"  );
  CXXTLS_LOG( "Program Output:" );
  gdb.printAndDeleteTargetOutput();
  CXXTLS_LOG( ">>>>>>>>>>>>>>>>>>>>>>>"  );


#endif


}


