
//
// Copyright 2002, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.



#include <cxxtls/hash_list.h>
#include <cxxtls/hashers.h>

#include <stdlib.h>
#include <portable_io.h>

using namespace std;
using namespace cxxtls;

typedef Hasher<int> IntHasher;


int main(int argc, char **argv)
{
  int TEST_ITEMS=1000 * 1000;

  if(argc > 1)
  {
    sscanf(argv[1], "%d", &TEST_ITEMS);
  }


  typedef hash_list<int,int,IntHasher> HL;
  typedef HL::mapped_type              mapped_type;
  typedef HL::iterator                 iterator;


  HL hl;

  int i;

  for(i=0; i < TEST_ITEMS; ++i)
    hl.insert( mapped_type(i,i) );


  iterator f = hl.begin();
  iterator l = hl.end();

  i=0;

  while(f != l)
  {
    int t = f->data_.first;

    if( t != i )
    {
      fprintf(stderr, "oops, data bad at i = %d", i);
      exit(1);
    }

    ++f;
    ++i;

  }

  if(i != TEST_ITEMS)
  {
      fprintf(stderr, "oops, iterators broken at %d", i);
      exit(1);
  }

  f = hl.find(10);

  if( f == l )
  {
    fprintf(stderr, "oops, couldn't find item '10'");
    exit(1);
  }

  for(i=0; i < TEST_ITEMS; ++i)
  {
    f = hl.find(i);

    if(f == l )
    {
      fprintf(stderr, "oops, couldn't find item '%d'", i);
      exit(1);
    }

  }

  for(i=0; i < TEST_ITEMS; ++i)
  {
    f = hl.find(i);

    if(f == l )
    {
      fprintf(stderr, "oops, couldn't find item '%d'", i);
      exit(1);
    }
    else
    {
      hl.erase(f);
    }

  }

  if(hl.size() != 0)
  {
    fprintf(stderr, "oops, should be no items in the hash list now\n");
    exit(1);
  }

  {
     for(i=0; i < TEST_ITEMS; ++i)
     {

	int t = hl[i] = i;
	
	if(t != i )
	{
	   fprintf(stderr, "oops, hl[%d] did not yield %d\n", i, i);
	   exit(1);
	}
	
     }

     for(i=0; i < TEST_ITEMS; ++i)
     {
	int t = hl[i];
	
	if(t != i )
	{
	   fprintf(stderr, "oops, hl[%d] did not insert %d\n", i, i);
	   exit(1);
	}
	
     }

     HL const &chl = hl;

     for(i=0; i < TEST_ITEMS; ++i)
     {
	int t = chl[i];
	
	if(t != i )
	{
	   fprintf(stderr, "oops, const hl[%d] did not locate %d\n", i, i);
	   exit(1);
	}
	
     }



  }



  exit(0);

}
