#include <iostream>
#include <cxxtls/lexical_cast.h>


using namespace std;

class Base {};
class Derived : public Base {};


int main()
{

    {
        cout << "Testing single character to string conversion" << endl;
        
        std::string value;
        
        value = cxxtls::lexical_cast<std::string>( 'c' );
        
        if(value != "c")
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected 'c' but found '" << value << "'" << endl;
          
        }

    }


    {
        cout << "Testing unsigned character to string conversion" << endl;
        
        std::string value;

        typedef unsigned char type;

        type v=9;
        
        value = cxxtls::lexical_cast<std::string>( v );
        
        if(value != "9")
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }

    }

    {
        cout << "Testing unsigned character=255 to string conversion" << endl;
        
        std::string value;

        typedef unsigned char type;

        type v=-1;
        
        value = cxxtls::lexical_cast<std::string>( v );
        
        if(value != "255")
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }

    }


    {
        cout << "Testing unsigned character to formatted string conversion" << endl;
        
        std::string value;
        std::string expected("11");

        typedef unsigned char type;

        type v=9;
        
        value = cxxtls::lexical_cast<std::string>( v, cxxtls::fmtspec('o') );
        
        if(value != expected)
        {
          cout << __FILE__ << ':' << __LINE__ 
          << ":   Error -- expected " << expected << " but found '" << value << "'" << endl;
          
        }

    }

    {
        cout << "Testing unsigned character to wide formatted string conversion" << endl;
        
        std::string value;
        std::string expected("   11");

        typedef unsigned char type;

        type v=9;
        
        value = cxxtls::lexical_cast<std::string>( v, cxxtls::fmtspec(5, 'o') );
        
        if(value != expected)
        {
          cout << __FILE__ << ':' << __LINE__ 
          << ":   Error -- expected " << expected << " but found '" << value << "'" << endl;
          
        }

    }



    {
        cout << "Testing short to string conversion" << endl;
        
        std::string value;

        typedef short type;

        type v=11;
        
        value = cxxtls::lexical_cast<std::string>( v );
        
        if(value != "11")
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }

    }

    {
        cout << "Testing short to formatted string conversion" << endl;
        
        std::string value;
        std::string expected("  11");

        typedef short type;

        type v=11;
        
        value = cxxtls::lexical_cast<std::string>( v, cxxtls::fmtspec(4) );
        
        if(value != expected)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }

    }

    {

        cout << "Testing short to formatted hex string conversion" << endl;
        
        std::string value;
        std::string expected("   b");

        typedef short type;

        type v=11;
        
        value = cxxtls::lexical_cast<std::string>( v, cxxtls::fmtspec(4,'x') );
        
        if(value != expected)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }

    }


    {
        cout << "Testing short to formatted string conversion with leading zeros" << endl;
        
        std::string value;
        std::string expected("0011");

        typedef short type;

        type v=11;
        
        value = cxxtls::lexical_cast<std::string>( v, cxxtls::fmtspec(4, cxxtls::fmtPad('0')) );
        
        if(value != expected)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }

    }


    {
        cout << "Testing short to string conversion" << endl;
        
        std::string value;

        typedef short type;

        type v=-32;
        
        value = cxxtls::lexical_cast<std::string>( v );
        
        if(value != "-32")
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected '" << v << "' but found '" << value << "'" << endl;
          
        }

    }

    {
        cout << "Testing short to formatted string conversion" << endl;
        
        std::string value;
        std::string expected("-32  ");

        typedef short type;

        type v=-32;
        
        value = cxxtls::lexical_cast<std::string>( v, cxxtls::fmtspec(-5) );
        
        if(value != expected)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected '" << expected << "' but found '" << value << "'" << endl;
          
        }

    }


    {
        cout << "Testing unsigned short to string conversion" << endl;
        
        std::string value;

        typedef unsigned short type;

        type v=-1;
        
        value = cxxtls::lexical_cast<std::string>( v );
        
        if(value != "65535")
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }

    }

    {
        cout << "Testing unsigned short to formatted string conversion" << endl;
        
        std::string value;

        typedef unsigned short type;

        type v=-1;
        
        value = cxxtls::lexical_cast<std::string>( v, cxxtls::fmtspec() );
        
        if(value != "65535")
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }

    }


    {
        cout << "Testing int to string conversion" << endl;
        
        std::string value;

        typedef int type;

        type v=-32;
        
        value = cxxtls::lexical_cast<std::string>( v );
        
        if(value != "-32")
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }

    }

    {
        cout << "Testing int to formatted string conversion" << endl;
        
        std::string value;   
        std::string expected("       -32");

        typedef int type;

        type v=-32;
        
        value = cxxtls::lexical_cast<std::string>( v, cxxtls::fmtspec(10) );
        
        if(value != expected)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected '" << expected << "' but found '" << value << "'" << endl;
          
        }

    }

    {
        cout << "Testing int to formatted string conversion with leading zeros" << endl;
        
        std::string value;   
        std::string expected("-000000032");

        typedef int type;

        type v=-32;
        
        value = cxxtls::lexical_cast<std::string>( v, cxxtls::fmtspec(10, cxxtls::fmtPad('0') ) );
        
        if(value != expected)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected '" << expected << "' but found '" << value << "'" << endl;
          
        }

    }



    {
        cout << "Testing unsigned int to string conversion" << endl;
        
        std::string value;

        typedef unsigned int type;

        type v=-1;
        
        value = cxxtls::lexical_cast<std::string>( v );
        
        if(value != "4294967295")
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }

    }

    {
        cout << "Testing unsigned int to formatted string conversion" << endl;
        
        std::string value;
        std::string expected("4294967295 ");

        typedef unsigned int type;

        type v=-1;
        
        value = cxxtls::lexical_cast<std::string>( v, cxxtls::fmtspec(-11) );
        
        if(value != expected)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }

    }



    {
        cout << "Testing long to string conversion" << endl;
        
        std::string value;

        typedef long type;

        type v=-32;
        
        value = cxxtls::lexical_cast<std::string>( v );
        
        if(value != "-32")
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }

    }

    {
        cout << "Testing long to formatted string conversion" << endl;
        
        std::string value;
        std::string expected("-32 ");

        typedef long type;

        type v=-32;
        
        value = cxxtls::lexical_cast<std::string>( v, cxxtls::fmtspec(-4) );
        
        if(value != expected)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }

    }


    {
        cout << "Testing unsigned long to string conversion" << endl;
        
        std::string value;

        typedef unsigned long type;

        type v=-1;
        
        value = cxxtls::lexical_cast<std::string>( v );
        
#ifdef _MSC_VER
        if(value != "4294967295")
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected '4294967295' but found '" << value << "'" << endl;
          
        }
#else
        if(value != "18446744073709551615")
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected '" << v << "' but found '" << value << "'" << endl;
          
        }
#endif

    }

#ifdef _MSC_VER
    {
        cout << "Testing unsigned long to formatted string conversion" << endl;
        
        std::string value; 
        std::string expected(" 4294967295");

        typedef unsigned long type;

        type v=-1;
        
        value = cxxtls::lexical_cast<std::string>( v, cxxtls::fmtspec(11) );
        
        if(value != expected)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }

    }
#else
    {
        cout << "Testing unsigned long to formatted string conversion" << endl;
        
        std::string value; 
        std::string expected(" 18446744073709551615");

        typedef unsigned long type;

        type v=-1;
        
        value = cxxtls::lexical_cast<std::string>( v, cxxtls::fmtspec(21) );
        
        if(value != expected)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }

    }
#endif


    {
        cout << "Testing long long to string conversion" << endl;
        
        std::string value;

        typedef long long type;

        type v=-32;
        
        value = cxxtls::lexical_cast<std::string>( v );
        
        if(value != "-32")
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }

    }

    {
        cout << "Testing unsigned long long to string conversion" << endl;
        
        std::string value;

        typedef unsigned long long type;

        type v=-1;
        
        value = cxxtls::lexical_cast<std::string>( v );
        
        if(value != "18446744073709551615")
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }

    }


    {
        cout << "Testing string to character conversion" << endl;

        std::string v("c");

        char value = cxxtls::lexical_cast<char>( v );
        
        if(value != 'c')
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }
    }


    {
        cout << "Testing string to unsigned character conversion" << endl;

        std::string v("255");

        char unsigned value = cxxtls::lexical_cast<unsigned char>( v );
        
        if(value != 255)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }
    }

    {
        cout << "Testing string to positive signed int conversion" << endl;

        std::string v("14");

        int value = cxxtls::lexical_cast<int>( v );
        
        if(value != 14)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }
    }

    {
        cout << "Testing string to negative signed int conversion" << endl;

        std::string v("-14");

        int value = cxxtls::lexical_cast<int>( v );
        
        if(value != -14)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }
    }


    {
        cout << "Testing string to long conversion" << endl;

        std::string v("3000000000");

        long value = cxxtls::lexical_cast<long>( v );

        long val = 3000;

        val *= 1000;
        val *= 1000;

        if(value != val)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }
    }

    {
        cout << "Testing string to unsigned long long conversion" << endl;

        std::string v("-1");

        unsigned long value = cxxtls::lexical_cast<unsigned long>( v );

        unsigned long val = -1;

        if(value != val)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }
    }


    {
        cout << "Testing string to long long conversion" << endl;

        std::string v("3000000000");

        long long value = cxxtls::lexical_cast<long long>( v );

        long long val = 3000;

        val *= 1000;
        val *= 1000;

        if(value != val)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }
    }

    {
        cout << "Testing string to unsigned long long conversion" << endl;

        std::string v("-1");

        unsigned long long value = cxxtls::lexical_cast<unsigned long long>( v );

        unsigned long long val = -1;

        if(value != val)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << v << " but found '" << value << "'" << endl;
          
        }
    }


    {
        cout << "Testing int to int conversion" << endl;

        int x = cxxtls::lexical_cast<int>(99);

        if(x != 99)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected " << 99 << " but found '" << x << "'" << endl;
        }
    }

    {
        cout << "Testing string to char conversion" << endl;

        char x = cxxtls::lexical_cast<char>(string("f"));

        if(x != 'f')
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected '" << 'f' << "' but found '" << x << "'" << endl;
        }

    }

    {
        cout << "Testing string to int conversion" << endl;

        int x = cxxtls::lexical_cast<int>(string("1234"));

        if(x != 1234)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected '" << 1234 << "' but found '" << x << "'" << endl;
        }

    }

    {
       bool exceptionThrown = false;

       try
       {
        cxxtls::lexical_cast<int>(string("a1234"));
       }
       catch(...)
       {
          exceptionThrown = true;
       }

       if(!exceptionThrown)
       {
          cout << __FILE__ << ':' << __LINE__ << ":  Error expected exception" << endl;
       }


    }
    {
       bool exceptionThrown = false;

       try
       {
        cxxtls::lexical_cast<char>(string("a1234"));
       }
       catch(...)
       {
          exceptionThrown = true;
       }

       if(!exceptionThrown)
       {
          cout << __FILE__ << ':' << __LINE__ << ":  Error expected exception" << endl;
       }


    }
    {
       bool exceptionThrown = false;

       try
       {
        cxxtls::lexical_cast<short>(string("a1234"));
       }
       catch(...)
       {
          exceptionThrown = true;
       }

       if(!exceptionThrown)
       {
          cout << __FILE__ << ':' << __LINE__ << ":  Error expected exception" << endl;
       }


    }
    {
       bool exceptionThrown = false;

       try
       {
        cxxtls::lexical_cast<unsigned>(string("a1234"));
       }
       catch(...)
       {
          exceptionThrown = true;
       }

       if(!exceptionThrown)
       {
          cout << __FILE__ << ':' << __LINE__ << ":  Error expected exception" << endl;
       }


    }
    {
       bool exceptionThrown = false;

       try
       {
        cxxtls::lexical_cast<long>(string("a1234"));
       }
       catch(...)
       {
          exceptionThrown = true;
       }

       if(!exceptionThrown)
       {
          cout << __FILE__ << ':' << __LINE__ << ":  Error expected exception" << endl;
       }


    }
    {
       bool exceptionThrown = false;

       try
       {
        cxxtls::lexical_cast<long long>(string("a1234"));
       }
       catch(...)
       {
          exceptionThrown = true;
       }

       if(!exceptionThrown)
       {
          cout << __FILE__ << ':' << __LINE__ << ":  Error expected exception" << endl;
       }


    }
    {
       bool exceptionThrown = false;

       try
       {
        cxxtls::lexical_cast<float>(string("a1234"));
       }
       catch(...)
       {
          exceptionThrown = true;
       }

       if(!exceptionThrown)
       {
          cout << __FILE__ << ':' << __LINE__ << ":  Error expected exception" << endl;
       }


    }

    {
       bool exceptionThrown = false;

       try
       {
        cxxtls::lexical_cast<double>(string("a1234"));
       }
       catch(...)
       {
          exceptionThrown = true;
       }

       if(!exceptionThrown)
       {
          cout << __FILE__ << ':' << __LINE__ << ":  Error expected exception" << endl;
       }


    }




    {
        cout << "Testing string to long conversion" << endl;

        long x = cxxtls::lexical_cast<long>("1234");

        if(x != 1234)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected '" << 1234 << "' but found '" << x << "'" << endl;
        }

    }

    {
        cout << "Testing string to short conversion" << endl;

        short x = cxxtls::lexical_cast<short>(string("1234"));

        if(x != 1234)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected '" << 1234 << "' but found '" << x << "'" << endl;
        }

    }

    {
        cout << "Testing string to  conversion" << endl;

        float x = cxxtls::lexical_cast<float>(string("0.5"));

        if(x != 0.5)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected '" << 0.5 << "' but found '" << x << "'" << endl;
        }

    }

    {
        cout << "Testing char const * to  conversion" << endl;

        float x = cxxtls::lexical_cast<float>("0.5");

        if(x != 0.5)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected '" << 0.5 << "' but found '" << x << "'" << endl;
        }

    }

    {
        cout << "Testing char const * to float conversion" << endl;

        float x = cxxtls::lexical_cast<float>("0.5");

        if(x != 0.5)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected '" << 0.5 << "' but found '" << x << "'" << endl;
        }

    }

    {
        cout << "Testing char const * to large float conversion" << endl;

        float x = cxxtls::lexical_cast<float>("3.4e38");

        if(x < 3.39e38 || x > 3.49e38)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected '" << 3.4e38 << "' but found '" << x << "'" << endl;
        }

    }


    {
        cout << "Testing double to float conversion" << endl;

        float x = cxxtls::lexical_cast<float>(double(0.5));

        if(x != 0.5)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected '" << 0.5 << "' but found '" << x << "'" << endl;
        }

    }


    {
        cout << "Testing double to string conversion" << endl;

        string expected("1e+100");

        string x = cxxtls::lexical_cast<string>(1e100);

        if(x != expected)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected '" << expected << "' but found '" << x << "'" << endl;
        }

    }

    {
        cout << "Testing double to string conversion" << endl;

        string expected("         1.0000e+100");

        string x = cxxtls::lexical_cast<string>(1e100,cxxtls::fmtspec(20,4, 'e'));

        if(x != expected)
        {
          cout << __FILE__ << ':' << __LINE__ << ":   Error -- expected '" << expected << "' but found '" << x << "'" << endl;
        }

    }




}
