
//
// Copyright 2002, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.



#include <cxxtls/sequence_map.h>
#include <iostream>
#include <stdlib.h>

using namespace std;
using namespace cxxtls;


typedef Sequence_Map<int,int>  SEQ_MAP;
typedef SEQ_MAP::sequence      SEQUENCE;
typedef SEQ_MAP::search        SEARCH;


int main()
{
   SEQ_MAP  map;


   // define the sequence, 1-2-3, and test everything you can test
   // with only one sequence in the system

   {
     SEQUENCE s1(map,  1);

     if(s1.can_complete())
     {
       cout << "dude, like can't complete an emtpy sequence!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     s1 += 1;
		   if(!s1.can_complete())
		   {
		     cout << "dude, like can't complete a trivial sequence!" << endl;
		     cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
		     exit(1);
		   }
     s1 += 2;
		   if(!s1.can_complete())
		   {
		     cout << "dude, like can't complete a trivial sequence!" << endl;
		     cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
		     exit(1);
		   }
     s1 += 3;

		   if(!s1.can_complete())
		   {
		     cout << "dude, like can't complete a trivial sequence!" << endl;
		     cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
		     exit(1);
		   }

     if(!s1.complete())
     {
       cout << "dude, like complete malfunction!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }
   }

   {
     SEARCH cmp(map);
     int id;

     if(cmp.finished(&id))
     {
       cout << "dude, like 'finished' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     if(!cmp(1))
     {
       cout << "dude, like 'search::operator()(T)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     ++cmp;

     if(!cmp(2))
     {
       cout << "dude, like 'search::operator()(T)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     ++cmp;

     if(!cmp(3))
     {
       cout << "dude, like 'search::operator()(T)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     ++cmp;

     id = 9;

     if(!cmp.finished(&id))
     {
       cout << "dude, like 'finished' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     if(id != 1)
     {
       cout << "dude, like 'finished' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     static int sequence[]=
     {
	 1,
	 2,
	 3
     };

     id=9;

     if(!map.find(sequence, 3, &id))
     {
       cout << "dude, like 'find(ptr,count,id)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     if(id != 1)
     {
       cout << "dude, like 'finished' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     id=9;

     if(map.find(sequence, 4, &id))
     {
       cout << "dude, like 'find(ptr,count,id)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }


     if(map.find(sequence, 1, &id))
     {
       cout << "dude, like 'find(ptr,count,id)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     if(map.find(sequence, 2, &id))
     {
       cout << "dude, like 'find(ptr,count,id)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     if(map.find(sequence, 0, &id))
     {
       cout << "dude, like 'find(ptr,count,id)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     //////////////////////////////////////////////////////
     id=9;
     if(!map.find(sequence, sequence+3, &id))
     {
       cout << "dude, like 'find(ptr,count,id)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }
     if(id != 1)
     {
       cout << "dude, like 'finished' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }


     if(map.find(sequence, sequence+4, &id))
     {
       cout << "dude, like 'find(ptr,count,id)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     if(map.find(sequence, sequence+1, &id))
     {
       cout << "dude, like 'find(ptr,count,id)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     if(map.find(sequence, sequence+2, &id))
     {
       cout << "dude, like 'find(ptr,count,id)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     if(map.find(sequence, sequence+0, &id))
     {
       cout << "dude, like 'find(ptr,count,id)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }
   }

/////////////////////////////////////////////////////////////////////////

   // define a new sequence, 1-2-4-5, and make sure that it passes all
   // the same kinds of tests as before and also that it doesn't break
   // the functions of the 1-2-3 sequence either.

   {
     SEQUENCE s1(map,  2);

     s1 += 1;
	       if(s1.can_complete())
	       {
		 cout << "dude, like can_complete() is broken!" << endl;
		 cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
		 exit(1);
	       }
     s1 += 2;
	       if(s1.can_complete())
	       {
		 cout << "dude, like can_complete() is broken!" << endl;
		 cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
		 exit(1);
	       }
     s1 += 4;
	       if(!s1.can_complete())
	       {
		 cout << "dude, like can_complete() is broken!" << endl;
		 cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
		 exit(1);
	       }
     s1 += 5;

     if(!s1.can_complete())
     {
       cout << "dude, like can't complete a trivial sequence!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     if(!s1.complete())
     {
       cout << "dude, like complete malfunction!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }
   }

   {
     SEARCH cmp(map);
     int id;

     if(cmp.finished(&id))
     {
       cout << "dude, like 'finished' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     if(!cmp(1))
     {
       cout << "dude, like 'search::operator()(T)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     ++cmp;

     if(!cmp(2))
     {
       cout << "dude, like 'search::operator()(T)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     ++cmp;

     if(!cmp(4))
     {
       cout << "dude, like 'search::operator()(T)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     ++cmp;

     if(!cmp(5))
     {
       cout << "dude, like 'search::operator()(T)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     ++cmp;

     id = 9;

     if(!cmp.finished(&id))
     {
       cout << "dude, like 'finished' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     if(id != 2)
     {
       cout << "dude, like 'finished' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     static int sequence[]=
     {
	 1,
	 2,
	 4,
	 5
     };

     id=9;
     if(!map.find(sequence, 4, &id))
     {
       cout << "dude, like 'find(ptr,count,id)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }
     if(id != 2)
     {
       cout << "dude, like 'finished' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     if(map.find(sequence, 5, &id))
     {
       cout << "dude, like 'find(ptr,count,id)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     if(map.find(sequence, 1, &id))
     {
       cout << "dude, like 'find(ptr,count,id)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     if(map.find(sequence, 2, &id))
     {
       cout << "dude, like 'find(ptr,count,id)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     if(map.find(sequence, 0, &id))
     {
       cout << "dude, like 'find(ptr,count,id)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     //////////////////////////////////////////////////////

     id=9;
     if(!map.find(sequence, sequence+4, &id))
     {
       cout << "dude, like 'find(ptr,count,id)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }
     if(id != 2)
     {
       cout << "dude, like 'finished' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }


     if(map.find(sequence, sequence+5, &id))
     {
       cout << "dude, like 'find(ptr,count,id)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     if(map.find(sequence, sequence+1, &id))
     {
       cout << "dude, like 'find(ptr,count,id)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     if(map.find(sequence, sequence+2, &id))
     {
       cout << "dude, like 'find(ptr,count,id)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     if(map.find(sequence, sequence+0, &id))
     {
       cout << "dude, like 'find(ptr,count,id)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }
   }


   {
     SEARCH cmp(map);
     int id;

     if(!cmp(1))
     {
       cout << "dude, like 'search::operator()(T)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     ++cmp;

     if(!cmp(2))
     {
       cout << "dude, like 'search::operator()(T)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     ++cmp;

     if(!cmp(3))
     {
       cout << "dude, like 'search::operator()(T)' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     ++cmp;

     id = 9;

     if(!cmp.finished(&id))
     {
       cout << "dude, like 'finished' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     if(id != 1)
     {
       cout << "dude, like 'finished' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }
   }

////////////////////////////////////////////////////////////////////


   {
     SEARCH cmp(map);

     int id;

     if(cmp.finished(&id))
     {
       cout << "dude, like 'finished' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     cmp(1);
     ++cmp;

     if(cmp.finished(&id))
     {
       cout << "dude, like 'finished' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     cmp(2);
     ++cmp;

     if(cmp.finished(&id))
     {
       cout << "dude, like 'finished' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }

     cmp(3);
     ++cmp;

     if(!cmp.finished(&id))
     {
       cout << "dude, like 'finished' malfunctioned!" << endl;
       cout << "      see " << __FILE__ << ":" << __LINE__ << endl;
       exit(1);
     }
   }

   exit(0);
}
