
//
// Copyright 2002, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.



#include <portable_new.h>
#include <portable_io.h>

// if you want lots of diagnostic output, #define DIAGNOSTIC.
// note that if you start printing the internal nodes of the
// skiprope, the test won't be stable from version of one version of the os to another
// so don't leave DIAGNOSTIC defined

#define SKIPROPE_MAX_BLOCKSIZE 32    /* 8 ints, that is important for the test */
#include <cxxtls/skiprope.h>

using namespace std;
using namespace cxxtls;

#define STARTUP_MESSAGE  "Skiprope testing"

static int nodes=0;

struct node_info { void *address; ptrdiff_t location; };

static node_info node[100000];

void addnode(void *n, ptrdiff_t location)
{
  node[nodes].address = n;
  node[nodes].location = location;
  ++nodes;
}

ptrdiff_t location(void *n)
{
  for(int i = 0; i < nodes; ++i)
  {
    if(node[i].address == n)
      return node[i].location;
  }

  printf("error:  can't find node %p\n", n);
  exit(1);

  return 0;

}

FILE *outputfile;
FILE *alternate_outputfile;
FILE *stdoutfile;


namespace cxxtls
{
template<class T>
class skiprope_debugger
//
//  this class defines a collection of routines that let you screw around
//  with skiprope members directly
//
{
  typedef typename skiprope<T>::node  node;

public:

  static void print_nodes(skiprope<T> &sr)
	 {
#ifdef DIAGNOSTIC
	  fprintf(outputfile,"skiprope<T> addr %p\n", (void*)(map_ptr(&sr)) );
	  fprintf(outputfile,"            size %ld\n", (long)(sr.size()));
	  fprintf(outputfile,"            lvls %d\n",  sr.active_levels_);
	  fprintf(outputfile,"               ");
	    {
	      for(int i = 0; i < sr.active_levels_; ++i)
	      {
		fprintf(outputfile,"%d={%p,%ld} ", i, (void*)(map_ptr(sr.level_[i].head_)), (long)(sr.level_[i].offset_));
	      }
	      fprintf(outputfile,"\n");
	    }
	  fprintf(outputfile,"            last %p\n",  (void*)(map_ptr(sr.last_)));
	  fprintf(outputfile,"            nods %d\n",  sr.nodes_);

	  ptrdiff_t node_start=0;
	  nodes=0;
	
	  for(node *n = sr.level_[0].head_; n; n = n->link_[0].next_)
	  {
	    addnode((void*)n, node_start);
	
	    fprintf(outputfile,"              node(%p) at(%ld) size(%ld) levels(%d)\n",
		    (void*)(map_ptr(n)),
		    (long)(node_start),
		    (long)(n->size()),
		    n->levels());
		

	    int i;
	    fprintf(outputfile,"                levels=");
	    for(i=0; i < n->levels(); ++i)
	    {
	      fprintf(outputfile,"{n=%p,p=%p,w=%ld} ",
		      (void*)(map_ptr(n->link_[i].next_)),
		      (void*)(map_ptr(n->link_[i].prev_)),
		      (long)(n->link_[i].width_)
		    );

	      if(i==0 && n->link_[i].width_ != n->size())
	      {
		fprintf(outputfile,"\n\nlevel 0 width_ != n->size()\n");
		exit(1);
	      }
	
	
	      if(i==0 && n->link_[i].width_ > skiprope<int>::chunk_size)
	      {
		fprintf(outputfile,"\n\nlevel 0 width_ > chunk_size\n");
		exit(1);
	      }


	      if(n->link_[i].prev_)
	      {
		ptrdiff_t old_location = location(n->link_[i].prev_);
		
		if(node_start - old_location != n->link_[i].prev_->link_[i].width_)
		{
		  fprintf(outputfile,"\n\nprev_ width broken\n");
		  exit(1);
		}
	      }
	
	      if(n->link_[i].next_)
	      {
		if(n->link_[i].next_->link_[i].prev_ != n)
		{
		  fprintf(outputfile,"\n\nnext_ is broken\n");
		  exit(1);
		}
	      }
	
	
	      if(sr.level_[i].head_ == n)
	      {
		if(node_start != sr.level_[i].offset_)
		{
		  fprintf(outputfile,"\n\nlevel_[i].offset_ wrong\n");
		  exit(1);
		}
	      }
	

	    }
	    fprintf(outputfile,"\n");
	
	    fprintf(outputfile,"                data=");
	    for(i=0; i < n->size(); ++i)
	    {
	      fprintf(outputfile,"%d ", n->data_[i]);
	    }
	    fprintf(outputfile, "\n");
	
	    node_start += n->size();
	  }
	
#endif
	 }

};
}


typedef skiprope_debugger<int> DEBUGGER;
skiprope_debugger<int> srid;


static int new_count=0;

#ifdef LINUX
#define OVERRIDE inline /* linux wierdness that prevents new/delete mismatch*/
#else
#define OVERRIDE
#endif


OVERRIDE void *operator new(size_t s)
  THROW(std::bad_alloc)
{
  if(s < 1)
    s = 0;

  ++new_count;

  void *rv = malloc(s);

  return rv;

}

OVERRIDE void operator delete(void* p)
  THROW_NOTHING
{
  if(p)
  {
    --new_count;
    free(p);
  }
}


void dump_int(skiprope<int> *r)
{
      DEBUGGER::print_nodes(*r);

}


int main(int argc, char **argv)
{


  int i;

  printf("skiprope test program (%s)\n", STARTUP_MESSAGE);
  fflush(stdout);

  outputfile = stdout;
  alternate_outputfile = fopen("/dev/null", "w");
  stdoutfile = stdout;

  if( argc > 1)
  {
    outputfile = alternate_outputfile;  // for debugging
  }

  new_count=0;

  { // perform tests in an arena where calls to 'new' can be counted

    skiprope<int> sri;

    // insert some test data vai push_back

    for(i=0; i < 100; ++i)
    {
#if 0
      DEBUGGER::print_nodes(sri);
#endif
       sri.push_back(i);

       if(sri.size() != i+1)
       {
	 printf("size() is broken at insert %d\n", i);
	 exit(1);
       }
    }

    DEBUGGER::print_nodes(sri);

    // verify find and operator[]

    if(sri.size() != 100)
    {
      printf("oops, size is wrong!\n");
      exit(1);
    }

    {
      for(i=0; i < 100; ++i)
      {
	if(sri[i] != i)
	{
	  printf("oops, sri[%d] is value %d!\n", i, sri[i]);
	  exit(1);
	}
      }
    }

    // verify forward iteration

    {
      skiprope<int>::iterator first = sri.begin();
      skiprope<int>::iterator last  = sri.end();

      for(i=0; first != last; ++first, ++i)
      {
	if(*first != i)
	{
	  printf("oops, iterator broken at %d, found %d\n",
		 i,
		 *first
		);
	  exit(1);
	}
      }

      if( i != 100)
      {
	printf("iteration stopped at location %d not %d\n",
		i, 100
	      );
	exit(1);
      }

    }

    // verify backwards iteration

    // verify forward iteration

    {
      skiprope<int>::iterator first = sri.begin();
      skiprope<int>::iterator last  = sri.end();

      for(i=100; first != last; )
      {
	--last;
	--i;
	
	if(*last != i)
	{
	  printf("oops, backwards iteration broken at %d, found %d\n",
		 i,
		 *last
		);
	  exit(1);
	}
      }

      if( i != 0)
      {
	printf("iteration stopped at location %d not %d\n",
		i, 0
	      );
	exit(1);
      }

    }


   // next test inserts at locations other than the end
  {
    skiprope<int>::iterator rv = sri.insert(sri.begin(), 100);

    if(sri.size() != 101)
    {
      printf("insert at front didn't get correct size\n");
      exit(1);
    }

    if( *rv != 0)
    {
      printf("insert returned the wrong address\n");
      exit(1);
    }

    DEBUGGER::print_nodes(sri);

    if(*sri.begin() != 100)
    {
      printf("insert(begin()) didn't work\n");
      exit(1);
    }

    skiprope<int>::iterator first = sri.begin();
    skiprope<int>::iterator last  = sri.end();

    ++first;
    i=0;
    while(first != last)
    {
      if(*first++ != i++)
      {
	printf("insert didn't get %d correct\n", i-1);
	exit(1);
      }
    }

    first = sri.begin();
    i=101;

    while(first != last)
    {
      --last;
      --i;
      int test_value;

      if(i == 0) 
        test_value = 100; 
      else 
        test_value = i-1;

      if(*last != test_value)
      {
	printf("backwards iteration after random insert is broken at %d\n",
	       i);
	exit(1);
      }

    }

    for(i=0; i < sri.size(); ++i)
    {
      if(i==0)
      {
	if(sri[i] != 100)
	{
	  printf("sri[0] is broken\n");
	  exit(1);
	}
      }
      else
      {
	if(sri[i] != i-1)
	{
	  printf("sri[%d] is broken (found %d not %d)\n", i, sri[i], i-1);
	  exit(1);
	}
      }
    }

    static int seven[]={107,106,105,104,103,102,101};

    rv = sri.insert(sri.begin(), seven, seven + 7);

    DEBUGGER::print_nodes(sri);

    if(*rv != 100)
    {
      printf("insert at front of seven didn't work right\n");
      exit(1);
    }

    if(sri.size() != 108)
    {
      printf("size is wrong when inserting seven\n");
      exit(1);
    }

    first = sri.begin();
    last  = sri.end();

    for( i=0; i < 8; ++i, ++first)
    {
      if(*first != 107 - i)
      {
	printf("insert seven at front didn't work\n");
	exit(1);
      }
    }

    for(i=0; i < 100; ++i)
    {
      if(*first++ != i)
      {
	printf("inserting seven at front screwed up 8-108\n");
	exit(1);
      }
    }

    if(first != last)
    {
       printf("iteration didn't reach end\n");
       exit(1);
    }

    for(i=0; i < 108; ++i)
    {
      if(i < 8)
      {
	if(sri[i] != 107-i)
	{
	  printf("sri[i] broken after insert of 7 at front\n");
	  exit(1);
	}
      }
      else
      {
	if(sri[i] != i-8)
	{
	  printf("sri[i] broken after insert of 7 at front\n");
	  exit(1);
	}
      }
    }

    {
      skiprope<int> x;
      int j;

      for(j=0; j < 1000; ++j)
      {
	rv = x.insert(x.begin(), j);
	
	if(j == 0)
	{
	  if(rv != x.end())
	  {
	    DEBUGGER::print_nodes(x);
	    printf("return value from insert at begin is wrong when empty\n");
	    exit(1);
	  }
	}
	else
	{
	  if(*rv != j-1)
	  {
	    DEBUGGER::print_nodes(x);
	    printf("return value, %d, from insert at %d isn't %d\n", *rv, j, j-1);
	    exit(1);
	  }
	}
	
      }
	
      DEBUGGER::print_nodes(x);
	
	
      if(x.size() != 1000)
      {
	printf("push front of a lot of little nodes is broken\n");
	exit(1);
      }

      for(j=0; j < 1000; ++j)
	if(x[j] != 999 - j)
	{
	  printf("push front of a lot of little nodes is broken\n");
	  exit(1);
	}

    }


    static int big[17] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16};
    
    if(sri.size() != 108)
    {
      printf("huh?  size changed....\n");
      exit(1);
    }

    DEBUGGER::print_nodes(sri);
    rv = sri.insert(sri.begin(), big, big + 17);

    DEBUGGER::print_nodes(sri);


    if(*rv != 107)
    {
      printf("insert of a big block didn't work\n");
      exit(1);

    }


    if(sri.size() != 125)
    {
      printf("insert of a big block didn't work\n");
      exit(1);
    }

    first = sri.begin();
    last  = sri.end();

    for(i=0; i < 17; ++i)
    {
      if(*first++ != i)
      {
	printf("iteration after big insert didn't work\n");
	exit(1);
      }
    }

    for(i=0; i < 8; ++i)
    {
      if(*first++ != 107-i)
      {
	printf("iteration after big insert didn't work\n");
	exit(1);
      }
    }

    for(i=0; i < 100; ++i)
    {
      if(*first++ != i)
      {
	printf("iteration after big insert didn't work\n");
	exit(1);
      }
    }

    if(first != last)
    {
	printf("iteration after big insert didn't work\n");
	exit(1);

    }

    for(i=0; i < 125; ++i)
    {
      if(i < 17)
      {
	if(sri[i] != i)
	{
	  printf("iteration after big insert didn't work\n");
	  exit(1);

	}
      }
      else if(i < 25)
      {
	if(sri[i] != 107 - (i-17))
	{
	  printf("iteration after big insert didn't work\n");
	  exit(1);
	}
      }
      else
      {
	if(sri[i] != i-25)
	{
	  printf("iteration after big insert didn't work\n");
	  exit(1);
	}
      }
    }

  }

  { // test random inserts of 1

    skiprope<int> y;

    int i;

    for(i=0; i < 100; ++i) y.push_back(1);

    DEBUGGER::print_nodes(y);

    while(y.size() < 10000)
    {


      unsigned location = y.size() * (unsigned)(rand());

      location /= RAND_MAX;

      if(location > (unsigned)(y.size()))
      {
	printf("bug in rand usage\n");
	exit(1);
      }

      skiprope<int>::iterator ptr = y.find(location);

      int old_size = y.size();
      y.insert(ptr, 1);

      if(y.size() != old_size + 1)
      {
	printf("random insert is breaking the size\n");
	exit(1);
      }

    }

    DEBUGGER::print_nodes(y);

    skiprope<int>::iterator ip = y.begin();

    long t=0;

    while(ip != y.end())
    {
      int value = *ip++;
      if(value != 1)
      {
	printf("skiprope<> contains bad data\n");
        exit(1);
      }
      t += value;
    }

    if(t != y.size())
    {
      printf("data in skiprope<> doesn't match size\n");
      exit(1);
    }

  }

  { // test random inserts of of a random number

    skiprope<int> y;

    int i;

    static int insert_data[17] =
    {
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
    };


    for(i=0; i < 100; ++i) y.push_back(1);

    DEBUGGER::print_nodes(y);

    while(y.size() < 10000)
    {

      unsigned location = y.size() * (unsigned)(rand());

      location /= RAND_MAX;

      if(location > unsigned(y.size()))
      {
	printf("bug in rand usage\n");
	exit(1);
      }


      static int rand_max = RAND_MAX;

      int size = 18 * (rand() >> 6);
      size /= (rand_max >> 6);

      if(size > 17)
         size=17;

      skiprope<int>::iterator ptr = y.find(location);

      int old_size = y.size();
      y.insert(ptr, insert_data, insert_data + size);

      if(y.size() != old_size + size)
      {
	printf("random insert of random size is breaking the size\n");
	exit(1);
      }

    }

    DEBUGGER::print_nodes(y);

    skiprope<int>::iterator ip = y.begin();

    for(i=0; i < y.size(); ++i)
    {
      if(y[i] != 1)
      {
	printf("data in y[] wrong\n");
	exit(1);
      }

      if(*ip != 1)
      {
	printf(" iterator to you is wrong\n");
	exit(1);
      }

      if(y.find(i) != ip)
      {
	printf("find or iteration broken\n");
	exit(1);
      }

      ++ip;


    }
  }

  { // simple erasures

    skiprope<int> y;

    int i;

    for(i=0; i < 10; ++i) y.push_back(i);

    while(y.size())
    {
      DEBUGGER::print_nodes(y);
      y.erase(y.begin());
    }

    for(i=0; i < 10; ++i) y.push_back(i);

    while(y.size())
    {
      DEBUGGER::print_nodes(y);

      skiprope<int>::iterator last = y.end();

      y.erase(--last);
    }

  } // simple erasures

  { // random erasures of 1

    skiprope<int> y;

    int i;

    for(i=0; i < 10000; ++i) y.push_back(i);

    while(y.size())
    {
      unsigned location = y.size() * (unsigned)(rand());
      location /= (unsigned)(RAND_MAX);

      if(location > (unsigned)(y.size()))
      {
	printf(" bug in rand usage\n");
	exit(1);
      }



      y.erase(y.find(location));

      if(y.size() == 100)
	DEBUGGER::print_nodes(y);
    }
  } // random erasures of 1

  }
  if(new_count != 0)
  {
    printf("new/delete mismatch (%d)\n", new_count);
    exit(1);
  }

  printf("skiprope testing ends with no errors detected\n");
  exit(0);
}











