
//
// Copyright 2002, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.



#include <cxxtls/streamable.h>

#include <iomanip>
#include <iostream>

using namespace std;
using namespace cxxtls;

class Derived
: public Ostreamable<Derived>
{
public:

    void ostreamable_helper(char c)
    {
      std::cout.write(&c,1);


    }
};

class ByteCounter
: public Ostreamable<ByteCounter>
{

public:

   int bytes_;

    ByteCounter()
    :  bytes_(0)
    {
    }



    void ostreamable_helper(char c)
    {
     ++bytes_;
    }


};



int main()
{

  Derived str;

  str << std::setw(8)
      << "one"
      << "."
      << std::endl;

  str << std::hex << 99 ;
  str << "."            ;
  str << 100            ;
  str << std::endl;


  StreamableString ss("stuff");

  ss
     << "--good "
     << std::hex
     << std::setw(12)
     << size_t(19)
     << "-eol"
     << std::endl;

  std::cout << ss;

  StreamableString ss_1(ss);

  std::cout << "the above should be copied here" << std::endl;

  std::cout << ss_1;

  std::cout << "next you should see 37" << std::endl;

  ss_1 = "37";

  std::cout << ss_1 << std::endl;

  {
      ByteCounter bc;

      for(int i = 0; i < 1000 * 1000; ++i)
      {
         bc << "\n";
      }

      if(bc.bytes_ != (1000 * 1000) )
      {
         std::cout << "ERROR:  bc.bytes_ is " << bc.bytes_ << ", not 1,000,000" << std::endl;
      }
      else
      {
        std::cout << "1,000,000 newlines came out" << std::endl;
      }


  }

  {
      ByteCounter bc;

      for(int i = 0; i < 100 * 1000; ++i)
      {
         bc << "012345678\n";
      }

      if(bc.bytes_ != (1000 * 1000) )
      {
         std::cout << "ERROR:  second bc.bytes_ is " << bc.bytes_ << ", not 1,000,000" << std::endl;
      }
      else
      {
        std::cout << "Another " << bc.bytes_ << " newlines came out" << std::endl;
      }


  }




  return 0;

}

