#include <cxxtls/thread.h>
#include <iostream>
#include <stdlib.h>
#include <string>

using namespace std;
using namespace cxxtls;


ThreadQueue<int>   commandQueue;

#ifdef _MSC_VER
 #define usleep(x) Sleep(x / 1000)    // Microsoft's Sleep() takes milliseconds not microseconds like usleep does
#endif

static void tryLockTest();


CXXTLS_DECLARE_THREAD_FUNCTION(enqueueInts,parm)
{
   commandQueue.push(0);   
   commandQueue.push(1);   
   commandQueue.push(2);   
   commandQueue.push(3);   

   commandQueue.push(-1);   // this value terminates the sumInts() below

   return 0;
}

static void multipleNotifiersTest();

CXXTLS_DECLARE_THREAD_FUNCTION(sumInts, parm)
{
     int sum=0;

     for(;;)
     {
         int tmp=0;
         commandQueue.waitPop(tmp);

         if(tmp < 0)
            break;

         sum += tmp;
        
     }

     CXXTLS_LOG("Data has been read, the integer sum is " << sum);

   
   return 0;
}




int main()
{

    // CXXTLS_SET_PROGRAM_TRACE_LEVEL(TT_HEAVYDEBUG);
    
    // CXXTLS_SET_PROGRAM_TRACE_LEVEL(TT_NONE);


// WARNING:  on Windows, all threads have to be joinable.

    ThreadId b = createThread(sumInts,   0);  // true => joinable thread

#ifndef _MSC_VER
    createThread(enqueueInts, 0, false);  // on unix they don't ALL have to be joinable
#else
    // on windows, all threads have to be joinable.
    ThreadId a = createThread(enqueueInts, 0);
#endif

#ifdef _MSC_VER
    joinThread(a);   // on windows they do.
#endif
    joinThread(b);

    CXXTLS_LOG("sumInts test complete");

    Mutex m;

    ScopedMutexLock lock(m);

    ConditionVariable cond;

    multipleNotifiersTest();


    tryLockTest();

    CXXTLS_LOG("Thread's test is complete");

}

struct NotifierParm
{
    Mutex mutex_;
    ConditionVariable condition_;

    bool doneFlag_;

    NotifierParm()
    {
        
    }

};

typedef std::pair<NotifierParm *, char const *> ThreadParm;

Mutex threadCountGuard;

int threadCount=0;  // 3 threads will be created for the multipleNotifiers test

CXXTLS_DECLARE_THREAD_FUNCTION(consumer, threadParm)
{

   ThreadParm *t = reinterpret_cast<ThreadParm*>(threadParm);

   std::string name = t->second;

   CXXTLS_LOG("Thread[" << name << "] begins");

   {
       ScopedMutexLock countLock(threadCountGuard);

       ++threadCount;
   }

   ScopedMutexLock lock(t->first->mutex_);

   t->first->condition_.wait(lock);

   CXXTLS_LOG("Thread[" << name << "] terminates");

   delete t;

  return 0;
}


static void multipleNotifiersTest()
{

   NotifierParm  parm;

   CXXTLS_LOG("Creating threads which will wait till notified");

   ThreadParm  tp;

   tp.first = &parm;
   
   tp.second = "thread1";

   ThreadId one = createThread(consumer, new ThreadParm(tp));

   // usleep(1000000);

   tp.second = "thread2";

   ThreadId two = createThread(consumer, new ThreadParm(tp));

   // usleep(1000000);

   tp.second = "thread3";

   ThreadId three = createThread(consumer, new ThreadParm(tp));

   CXXTLS_LOG("Waiting till threads have had a chance to start");

   for(;;)
   {
       {
          ScopedMutexLock lock(threadCountGuard);

          if(threadCount == 3)
             break;
       }
       usleep(1000);  // don't burn up the cpu
   }

   CXXTLS_LOG("Broadasting wakeup signal to all threads");

   parm.condition_.notifyAll();

   CXXTLS_LOG("Joining all threads");

   joinThread(one);
   joinThread(two);
   joinThread(three);
   
   CXXTLS_LOG("multipleNotifiersTest complete");

    
}

static Mutex tryMutex;

CXXTLS_DECLARE_THREAD_FUNCTION(tryLocker, threadParm)
{
    tryMutex.lock();

    usleep(2000000);

    tryMutex.unlock();

    return 0;
}


CXXTLS_DECLARE_THREAD_FUNCTION(tryTrier, threadParm)
{
    int misses = 0;
    int hits   = 0;

       usleep(100000);

    for(int i = 0; i < 10; ++i)
    {
       bool locked = tryMutex.tryLock();

       if(locked)
       {
          ++hits;
          tryMutex.unlock();
       }
       else
          ++misses;

       usleep(100000);
    }

    if(hits)
    {
        CXXTLS_LOG("ERROR:  we should not have been able to lock even once!  Found " << hits << " locks");
    }
    else
    if(misses < 7)
    {
        CXXTLS_LOG("ERROR:  we should have be been able to lock at least some times.   Found " << misses << " lockFails");
    }
    else
    {
        CXXTLS_LOG("tryLock test succeeded");
    }
       
    return 0;
}


static void tryLockTest()
{
    ThreadId a = createThread(tryLocker, 0);  
    ThreadId b = createThread(tryTrier,  0);  

    joinThread(a);
    joinThread(b);
    
}
