#include <cxxtls/varobj.h>

#include <stdlib.h>
#include <iostream>

#define bomb(m) \
        { cout << __FILE__ << ':' << __LINE__ << ":  Error:  expected " << m << endl; \
          exit(1); \
        }

#define CHECK(e) \
    if(!(e)) \
    { \
        bomb(#e); \
    } \
    else \
    { \
        cout << "Correct:  " << #e << endl; \
    }

using namespace std;
using namespace cxxtls;

int main()
{
     StringVarObj  svo("fred");

     CHECK(svo.value() == "fred");

     IntegerVarObj ivo(123);

     CHECK(ivo.value() == "123");

     FloatVarObj fvo(2.5);

     CHECK(fvo.value() == "2.5");

     {
         VarObj::ParseResult rv = VarObj::parse("\"stuff\"");

         if(rv.rv_ == 0)
         {
           string msg = "StringVarObj did not parse -- '";
           msg += rv.error_;
           msg += "'";

            bomb(msg);
         }

         CHECK(rv.rv_->value() == "stuff");

     }


     {
         VarObj::ParseResult rv = VarObj::parse("-123");

         if(rv.rv_ == 0)
         {
           string msg = "IntegerVarObj did not parse -- '";
           msg += rv.error_;
           msg += "'";

            bomb(msg);
         }

         CHECK(rv.rv_->value() == "-123");


     }

     {
         VarObj::ParseResult rv = VarObj::parse("-16.5");

         if(rv.rv_ == 0)
         {
           string msg = "FloatVarObj did not parse -- '";
           msg += rv.error_;
           msg += "'";

            bomb(msg);
         }

         CHECK(rv.rv_->value() == "-16.5");

     }

     {
         VarObj::ParseResult rv = VarObj::parse("{a=1, b=2, c={d=\"xyz\",e=4}}");

         if(rv.rv_ == 0)
         {
           string msg = "CompositeVarObj did not parse -- '";
           msg += rv.error_;
           msg += "'";

            bomb(msg);
         }

         cout << "Checking:  " << rv.rv_->value() << endl;

         CHECK(rv.rv_->value() == "{a=1,b=2,c={d=\"xyz\",e=4}}");

         {
             cout << "Checking composite member values:\n";
             cout << "   a:  ";
    
             VarObj* aPtr = rv.rv_->findChild("a");
    
             CHECK(aPtr != 0);
    
             CHECK(aPtr->value() == "1");
         }

         {
             cout << "   b:  ";
    
             VarObj* aPtr = rv.rv_->findChild("b");
    
             CHECK(aPtr != 0);
    
             CHECK(aPtr->value() == "2");
         }

         {
             cout << "   c:  ";
    
             VarObj* aPtr = rv.rv_->findChild("c");
    
             CHECK(aPtr != 0);

             CHECK(dynamic_cast<CompositeVarObj*>(aPtr) != 0);

             VarObj* dPtr = aPtr->findChild("d");

             CHECK(dPtr != 0);

             CHECK(dPtr->value() == "xyz");



    
         }


     }




     cout << "varobj_test passed!" << endl;
}


