
//
// Copyright 2002, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.



#include <cxxtls/cursorwindow.h>
#include <iostream>
#include <portable_io.h>
#include <cxxtls/cursesinterface.h>

using namespace std;
using namespace cxxtls;

class RepaintHandler
: public CursorWindow::viewport::repaint_handler
{
public:

   void operator() (CursorWindow::viewport* w, int cmd)
   {
     // paint the window

      CursorWindow::row_col size = w->size();

      w->set_text_attribute(CursorWindow::normal);

      if(size.row_ < 1 || size.row_ > 200)
	cout << "window rows is way off" << endl;

      if(size.col_ < 1 || size.col_ > 200)
	cout << "window cols is way off" << endl;

      int i;

      for(i=0; i < size.row_; ++i)
      {
	 char buffer[40];

	 sprintf(buffer, "row %d", i);

	 w->set_curpos(i,0);

	 w->write(buffer);

      }


      w->set_text_attribute(CursorWindow::reversed);

      w->set_curpos(0,0);
      w->write("*");
      w->set_curpos(size.row_-1, 0);
      w->write("*");
      w->set_curpos(0, size.col_-1);
      w->write("*");
      w->set_curpos(size.row_-1, size.col_-1);
      w->write("*");

      w->set_curpos(10,10);

   }
};


class BackgroundHandler
: public CursorWindow::viewport::repaint_handler
{
public:

   void operator() (CursorWindow::viewport* w, int cmd)
   {
     w->set_curpos(0,0);
     w->set_text_attribute(CursorWindow::normal);
     w->fill_to_eos('.');

     w->set_curpos(0,40);
     w->write("Press space to terminate");
     w->set_curpos(1,40);
     w->write("Press 'b' to activate the background window");
     w->set_curpos(2,40);
     w->write("Press 'w' to activate the foreground window");
     w->set_curpos(3,40);
     w->write("Press other keys to get simple outputs");


   }
};

class ResizeHandler
: public CursorWindow::resize_handler
{
public:

  CursorWindow::viewport *background_;

  void operator() (CursorWindow*w)
  {
    background_->move( CursorWindow::row_col(0,0), w->size() );
  }

};

typedef std::auto_ptr<CursorWindow::viewport> viewport_handle;

int main()
{
  CursorWindow window;

  window.open();

  RepaintHandler h;
  BackgroundHandler bh;

  viewport_handle   b( window.new_viewport(&bh).release() );
  viewport_handle   w( window.new_viewport(&h).release() );

  w->activate();

  ResizeHandler resizer;

  resizer.background_ = b.get();  // window resizes change the size of the
				  // background viewport


  window.set_resize_handler(&resizer);

  w->move(CursorWindow::row_col(5,10),
	  CursorWindow::row_col(15,30)
	 );

  {
    char buffer[100];

    sprintf(buffer, "viewport locations is %d, %d and it size is %d, %d",
	    w->origin().row_,
	    w->origin().col_,
	    w->size().row_,
	    w->size().col_
	   );
	
    window.set_curpos(0,0);
    window.set_text_attribute(CursorWindow::normal);
    window.write(buffer);

  }

  static int eCount=0;


  for(;;)
  {
    CursorWindow::input_event e = window.read_input();

    ++eCount;

    if(e.type_ == CursorWindow::input_event::ResizeKey)
    {
       h(w.get(),0);
       continue;
    }
    else
    if(e.type_ == CursorWindow::input_event::MouseEvent)
    {

      char buffer[40];

      sprintf(buffer, "Mouse(e=%02x, r=%02d, c=%02d)", e.value_, e.row_, e.col_);

      w->set_curpos(10,7);

      w->write(buffer);
      w->fill_to_eol(' ');


    }
    else
    {
  
      if(e.value_ == ' ')
        break;
  
      if(e.value_ == 'b')
        b->activate();
      else
      if(e.value_ == 'w')
        w->activate();
  
      w->set_curpos(10,10);
      w->write("                 ");
      w->set_curpos(10,10);
  
      
      
      w->write(CursorWindow::key_name(e.value_));
      
      if(e.value_ >= ' ' && e.value_ < 0x7f)
      {
        w->set_curpos(11,11);
        w->fill_to_eol(e.value_);
      }
      
      if(e.value_ >= ' ' && e.value_ < 0x7f)
      {
        w->set_curpos(12,12);
        w->fill_to_eos(e.value_);
      }
      
    }

    {
      char buffer[40];

      sprintf(buffer, "%03d", int(eCount));

      w->set_curpos(0,10);
      w->write(buffer);

    }


  }

}
