#ifndef CSV_VIEWER_HEADER_INCLUDED_p
#define CSV_VIEWER_HEADER_INCLUDED_p
//
// Copyright 2010, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.

#include <cxxtls/tableeditor.h>

namespace cxxtls
{

class CsvViewer
: public TableEditor
    ///
    ///  A Comma Separated Values file viewer.
    ///
    ///  This class reads and displays comma separated values files.
    ///  The user is allowed to change the contents of the file 
    ///  and save those changes or write them to a different file.
    ///
{
    std::string filename_;
    std::string fileError_;

public:

    CsvViewer(ViewerManager *vm,
              std::string const &filename, 
              int delimChar=','
             );
        ///<  Construct a CsvViewer and open its file and read
        ///   it into memory.  The lines in the file are 
        ///   assumed to be delimited by the specified delimiter character.
        ///   This constructor assumes that the first line of the file
        ///   contains the names of the columns rather than the
        ///   data for a row of text.  The column headers must be delimited
        ///   as well.
        ///   
        ///   @param vm       The viewermanager controlling this viewer.
        ///   
        ///   @param filename defines the file that contains the comma
        ///                   separated values.
        ///
        ///   @param delimChar defines the character that delimits the
        ///                    values -- the default is of course comma
        ///                    but other characters can be used -- tab,
        ///                    "|" or others as needed.
        ///   
    
    bool ok() const { return fileError_.empty(); }
        ///<  Return true if no file read errors have occurred.
        ///   See error() for details of any error.
    
    std::string const &error() const { return fileError_; }
        ///<  Returns the most recent file i/o error.
    
    virtual ~CsvViewer();



  virtual std::string const &application_name() const; ///<  Return the application name
  virtual std::string description() const;             ///<  Return a description of the application
  static Viewer* app(ViewerManager*vm, std::string &fullname);
    ///< Application "hook" function for the CsvViewer class.
    ///  Use it as a parameter to add_app, like this:
    ///  @code
    ///   vm.add_app(CsvViewer::app, "CsvViewer");
    ///  @endcode

private:

    virtual bool handle_event( CursorWindow::input_event const * e,
                             CursorWindow::viewport          * vp
                           );
        ///< Handle keystrokes and other events by superceding the
        ///  the behavior of the TableViewer.


   virtual void setCursorInfo(CursorWindow::viewport *vp,
                              size_t worldRow, 
                              size_t worldCol, 
                              bool displayed
                             );
       ///<  Superseed of base class version only for the purpose of detecting
       ///   field changes so that edit sessions on fields can be terminated.
       ///   @see TableViewer::setCursorInfo


    void saveTo(std::string const &filename); 
        ///< Write the table to the specified file.

    bool okToQuit(CursorWindow::viewport *vp);  
        ///<  Return true if it is ok to exit

     virtual void help();  ///<  Display help about this application

};
} // namespace cxxtls
#endif
