#ifndef cursesinterface_h_included
#define cursesinterface_h_included

//
// Copyright 2002, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.


///@file
///
///  Defines routines used to draw windows and text in cursor windows.

#include <cxxtls/file.h>

namespace cxxtls
    /// The namespace that encapsulates most of the functionality in the
    /// CXX toolkit.  
{

class CursorWindow;

namespace CursesInterface
   ///
   /// A namespace that wraps the many functions that interact with the
   /// curses terminal used by the program.
   ///
{
  void make_xterm_terminfo_file(FileName const &dir);
  void openCursesTerminal();
  void closeCursesTerminal();
  void getCursesCursor(int *row, int *col);
  void getCursesScreenSize(int *row, int *col);
  void setCursesAttributes( int a );
  void paintCharString(char const *r, int count, int a, int row, int col);
  void paintCharString(long c, int count, int a, int row, int col);
  void refreshCursesWindow();
  void beepCursesTerminal();
  void moveCursesCursor(int row, int col);
  void make_xterm_terminfo_file(FileName const &dir);

  extern bool winch_occurred;
  extern bool resize_requested;

  int read_mapped_key(CursorWindow&);  

  bool kbhit(CursorWindow&);   ///< returns true if at least one key is waiting
                               ///  to be read using read_mapped_key

  enum special_keys            /// non-keys treated as keys with wierd key values
  {
    MouseEvent = 0x8000000,    ///< a mouse event has occurred instead of a key being pressed
    ResizeEvent= -1            ///< a window resize event has occurred instead of a key being pressed
  };

  enum mouse_down_bits                 // names for the buttons 
  {
    left =1,
    right=2,
    middle=4
  };

  struct mouse_info
    /// Used in mouse events to describe the position and buttons
    /// (Actually, not implemented at this time).
  {
    // upper left corner is 0,0

    mouse_info(int row, int col, int mouseDownBitMask)
    :  row_(row),
       col_(col),
       state_(mouseDownBitMask)
    {
    }

    int row_;    //<  0 based row where mouse event occurred
    int col_;    //<  0 based column where mouse event occurred
    int state_;  //<  an or'ing of left, middle, and right -- implying those buttons are down
  };

  mouse_info read_mouse_info(); // only valid after read_mapped_key returns MouseUp

  enum line_drawing_characters   
    /// logical values for asic representations of box and line fragments
  {
    UL_CORNER,   ///<  upper left corner
    LL_CORNER,   ///<  lower left corner
    UR_CORNER,   ///<  upper right corner
    LR_CORNER,   ///<  lower right corner
    HL_MIDDLE,   ///<  horizontal line (middle of char chell)
    VL_MIDDLE,   ///<  verticlal line (middle of char cell)
    PL_BOX   ,   ///<  big plus sign
    TE_LEFT  ,   ///<  tee pointing right
    TE_RIGHT ,   ///<  tee pointing left
    TE_BOTTOM,   ///<  tee pointing up
    TE_TOP   ,   ///<  tee pointing down

    lcCOUNT // must be LAST!
  };

  extern long line_chars[lcCOUNT]; ///< a buffer holding the ASCII values of the line drawing characters

};

} // namespace cxxtls
#endif
