#ifndef	ftpviewer_included_h
#define	FTPviewer_included_h
//
// Copyright 2002, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.
//
// Warning:  not all files in this directory structure are covered by the
// same copyright.  Some of them are part of the GNU source distribution
// and you must obey the GPL copyright for those files.

///@file
///
/// This file defines the interface to the FTP viewer program, FTPviewer.

#include <cxxtls/viewer.h>

namespace cxxtls
{

class FileName;

class FTPviewer
: public Viewer
///
///   An FTPviewer is a Viewer meant for displaying and editing files    
///   and directories exported by an ftp server.   It a large extent, it looks
///   and feels like a directory viewer -- except that the files and directories
///   being displayed are on a remote system.  The ftp viewer also maintains a
///   reference to a 'current directory'	on the local machine where files on
///   the remote machine will be stored when you 'get' a file.  Analogously,
///   files in the local directory may be 'put' to the remote system.
/// 
///   When creating an FTPviewer, you must supply:
/// 
///   @li The name         of the remote machine
///   @li The user         id to use on the remote machine
///   @li The password the remote machine expects for that user.
/// 
///   You may only get, put, and navigate directories on the remote machine.
///   
///   To create an ftp viewer, use the static member function,
/// 
///   @code
///     FTPviewer::app(ViewerManager*, std::string local_directory)
///   @endcode
/// 
///   It will prompt the user for the needed information.  You must supply
///   the local directory as a parameter.
//
{

public:

  struct Impl;
    /// implemenation data not needing to be visible to users.

  std::auto_ptr<Impl> impl_; ///< Pointer to the implementation data

  FTPviewer(ViewerManager    *	   vm, 
	    FileName	const&	   local_directory,
	    std::string	const&	   remote_host,
	    std::string	const&	   userid,
	    std::string	const&	   password,
	    std::string	const&	   remote_directory=std::string()
	   );

 ~FTPviewer();

  virtual void operator() ( CursorWindow::viewport *     vp,
			    int                          cmd
			  );
     /// Handle screen repaint commands.
     ///
     ///  @param vp
     ///  @param cmd

  virtual bool handle_event( CursorWindow::input_event const * event,
			     CursorWindow::viewport          * viewport
			   );
     /// Handle input events.
     ///
     /// @param event 
     /// @param viewport

  virtual void help();  ///< Present the user with help information that is
			///  specific to this viewer's operation and contains
			///  information about system keep bindings, etc.

  static Viewer* app(ViewerManager*vm, std::string &fullname);
    //
    ///  Application "hook" function for the DirectoryViewer class.
    ///  Use it as a parameter to add_app, like this:
    ///
    ///  @code
    ///    vm.add_app(DirectoryViewer::app, "DirectoryViewer");
    ///  @endcode

  static std::string app_name;
    //
    ///  The name to prompt the user with in the ViewerManager::appChooser
    ///  menu.

  std::string const& application_name() const { return app_name; }
    //
    ///  Return the name of the application to use when describing this
    ///  viewer in the ViewerManager::appChooser or in the
    ///  ViewerManager::ViewerSelector.

  std::string description() const;
    //
    ///	 Returns a description of this session.


};
} // namespace cxxtls
#endif
