//
// Copyright 2002, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.
//
// Warning:  not all files in this directory structure are covered by the
// same copyright.  Some of them are part of the GNU source distribution
// and you must obey the GPL copyright for those files.


///@file
/// Implements members of the KeyViewer class which lets you see the
/// hex values of keys pressed by the user.

#include <cxxtls/keyviewer.h>

#include <iomanip>

namespace cxxtls
{

std::string KeyViewer::app_name = "KeyViewer";

bool KeyViewer::is_marked(size_t line) { return 0; }

void
KeyViewer::
handle_resize(CursorWindow::row_col const &vp_size)
{
  // ignore this
}

int
KeyViewer::
handle_event(CursorWindow::input_event const *e,
             size_t index,
             ViewerManager* vm
            )
{
  if(e->type_ == CursorWindow::input_event::ForceExitKey)
    return 2;  // terminate window


  using namespace std;

  StreamableString text;

  text << "Key [" << std::setw(3) << strings_.size() << "] "
       << "name is " << std::setw(12) << CursorWindow::key_name(e->value_) << " and"
       << " the key value is 0x" << std::setbase(16) << e->value_ << ", and the key type is "
       << e->type_ << '.';

  strings_.push_back(text);

  if(e->value_ == ('K' - '@') )
    return 2;

  return 1;
}


std::auto_ptr< std::list<std::string> >
KeyViewer::
help_text()
{
  std::auto_ptr< std::list< std::string > >rv(new std::list<std::string>);

  rv->push_back(" ");
  rv->push_back("Help for the KeyViewer.");
  rv->push_back(" ");
  rv->push_back("The Key Viewer exists to let you see the key code values as you press them.");
  rv->push_back(" ");
  rv->push_back("However, it cannot show you the key codes for 'System Comand Keys'.");
  rv->push_back(" ");
  rv->push_back("To see system command key values, press F1 or ^X-^H to get help.");
  rv->push_back(" ");
  rv->push_back("Otherwise, press keys and see their values.");
  rv->push_back(" ");
  rv->push_back("You can press Page Down and Page Up to scroll through");
  rv->push_back("new and previously entered values.");


  return rv;
}

Viewer*
KeyViewer::
app(ViewerManager*vm, std::string &fullname)
{
  KeyViewer *com = new KeyViewer();

  com->strings_.push_back("");
  com->strings_.push_back("Press KEYS to see what keycode they generate");
  com->strings_.push_back("");
  com->strings_.push_back("Except for 'System Comand Keys' which cannot be displayed.  See HELP info.");
  com->strings_.push_back("");
  com->strings_.push_back("If a single key generates multiple keycodes then your terminfo is wrong");
  com->strings_.push_back("and you should rm -fr ~/.tools_home/<hostname.*> directories, and let");
  com->strings_.push_back("a subsequent re-invocation of edit or show fix the problem for you.");
  com->strings_.push_back("");
  com->strings_.push_back("PRESS ^k TO GET OUT OF THIS WINDOW!!!!  (It's 0x0b in case you care)");
  com->strings_.push_back("");
  

  return new ListViewer(com,vm);
}

KeyViewer::
KeyViewer()
{
  title_ = "Key value displayer (press ^K to quit or F1 for help)";
}


KeyViewer::
~KeyViewer()
{
}

} // namespace cxxtls
